/**
 * @class Sb.Application
 * @extends Ext.util.Observable
 * @singleton
 */
Ext.define('Sb.Application', {
	extend: 'Ext.util.Observable',

	/**
	 * Last time when session was updated
	 */
	renewSessionHandlerLastPolling: 0,

	/**
	 * Do we renew external application`s (usually a Panel) session?
	 */
	isExternalSessionRenewed: false,

	constructor: function (config) {
		config = config || {};
		Ext.apply(this, config);
		this.addEvents(
			/**
			 * @event renewsession
			 * Fires when the need application session prolongation
			 */
			'renewsession'
		);
		Sb.Application.superclass.constructor.call(this, config);
		Ext.getBody().on('mousemove', this.renewSessionHandler, this);

		window.onerror = function (message, url, line) {
			Sb.SiteService.log('JavaScript error: ' + message + ' on line ' + line + ' for ' + url, 'error');
		};
	},

	/**
	 * Renew SiteBuilder and Control Panel sessions
	 */
	renewSessionHandler: function () {
		var now = new Date().getTime();
		if (now - this.renewSessionHandlerLastPolling > Sb.Config.renewSessionPeriod) {
			this.renewSessionHandlerLastPolling = now;
			Sb.SystemService.renewSession();
			if (this.isExternalSessionRenewed) {
				this.fireEvent('renewsession');
			}
		}
	}
});
