/**
 * @class Sb.FontManager
 * Utility class for fonts
 * @singleton
 */
Ext.define('Sb.FontManager', function () {
	var fontGroups = {
			sansSerif: 1,
			serif: 2,
			mono: 3,
			boldBlock: 4,
			decorative: 5,
			handwritten: 6,
			symbols: 7
		},
		fontFamily = {
			sansSerif: 'sans-serif',
			serif: 'serif',
			monospace: 'monospace',
			cursive: 'cursive',
			fantasy: 'fantasy'
		},

		external = 'external',
		internal = 'internal',

		fontFamilies = {},
		loadedFonts = {},

		externalFontsExclude = function (record) {
			return record[2] === internal;
		},

		defaultFonts = [
			['Arial', 				fontGroups.sansSerif,	internal, fontFamily.sansSerif],
			['Arial Black',			fontGroups.sansSerif,	internal, fontFamily.sansSerif],
			['Tahoma',				fontGroups.sansSerif,	internal, fontFamily.sansSerif],
			['Trebuchet MS',		fontGroups.sansSerif,	internal, fontFamily.sansSerif],
			['Verdana',				fontGroups.sansSerif,	internal, fontFamily.sansSerif],

			['Book Antiqua',		fontGroups.serif,		internal, fontFamily.serif],
			['Georgia',				fontGroups.serif,		internal, fontFamily.serif],
			['Terminal',			fontGroups.mono,		internal, fontFamily.monospace],
			['Times New Roman',		fontGroups.serif,		internal, fontFamily.serif],

			['Courier New',			fontGroups.mono,		internal, fontFamily.monospace],
			['Impact',				fontGroups.boldBlock,	internal, fontFamily.sansSerif],
			['Comic Sans MS',		fontGroups.handwritten,	internal, fontFamily.sansSerif],
			['Symbol',				fontGroups.symbols,		internal],
			['Webdings',			fontGroups.symbols,		internal],
			['Wingdings',			fontGroups.symbols,		internal]
		],
		latinFonts = [
			['Armata',				fontGroups.sansSerif,	external, fontFamily.sansSerif],
			['Dosis',				fontGroups.sansSerif,	external, fontFamily.sansSerif],
			['Duru Sans',			fontGroups.sansSerif,	external, fontFamily.sansSerif],
			['Oxygen',				fontGroups.sansSerif,	external, fontFamily.sansSerif],
			['Pontano Sans',		fontGroups.sansSerif,	external, fontFamily.sansSerif],
			['Source Sans Pro',		fontGroups.sansSerif,	external, fontFamily.sansSerif],
			['Varela',				fontGroups.sansSerif,	external, fontFamily.sansSerif],

			['Alegreya',			fontGroups.serif,		external, fontFamily.serif],
			['Caudex',				fontGroups.serif,		external, fontFamily.serif],
			['Cutive',				fontGroups.serif,		external, fontFamily.serif],
			['Habibi',				fontGroups.serif,		external, fontFamily.serif],
			['Headland One',		fontGroups.serif,		external, fontFamily.serif],
			['Noticia Text',		fontGroups.serif,		external, fontFamily.serif],

			['Abril Fatface',		fontGroups.boldBlock,	external, fontFamily.cursive],
			['Lilita One',			fontGroups.boldBlock,	external, fontFamily.cursive],
			['Passion One',			fontGroups.boldBlock,	external, fontFamily.cursive],
			['Righteous',			fontGroups.boldBlock,	external, fontFamily.cursive],
			['Titan One',			fontGroups.boldBlock,	external, fontFamily.cursive],

			['Bitter',				fontGroups.decorative,	external, fontFamily.serif],
			['Glass Antiqua',		fontGroups.decorative,	external, fontFamily.cursive],
			['Gruppo',				fontGroups.decorative,	external, fontFamily.cursive],
			['Overlock',			fontGroups.decorative,	external, fontFamily.cursive],
			['Poiret One',			fontGroups.decorative,	external, fontFamily.cursive],
			['Quando',				fontGroups.decorative,	external, fontFamily.serif],
			['Quattrocento',		fontGroups.decorative,	external, fontFamily.serif],
			['Simonetta',			fontGroups.decorative,	external, fontFamily.cursive],

			['Euphoria Script',		fontGroups.handwritten,	external, fontFamily.cursive],
			['Meie Script',			fontGroups.handwritten,	external, fontFamily.cursive],
			['Oleo Script',			fontGroups.handwritten,	external, fontFamily.cursive],
			['Parisienne',			fontGroups.handwritten,	external, fontFamily.cursive],
			['Playball',			fontGroups.handwritten,	external, fontFamily.cursive]
		],
		cyrillicFonts = [
			['Andika',				fontGroups.sansSerif,	external, fontFamily.sansSerif],
			['Anonymous Pro',		fontGroups.sansSerif,	external, fontFamily.sansSerif],
			['Open Sans',			fontGroups.sansSerif,	external, fontFamily.sansSerif],
			['PT Sans Caption',		fontGroups.sansSerif,	external, fontFamily.sansSerif],
			['Tenor Sans',			fontGroups.sansSerif,	external, fontFamily.sansSerif],
			['Ubuntu',				fontGroups.sansSerif,	external, fontFamily.sansSerif],

			['Forum',				fontGroups.serif,		external, fontFamily.cursive],
			['Ledger',				fontGroups.serif,		external, fontFamily.serif],
			['PT Serif',			fontGroups.serif,		external, fontFamily.serif],

			['Comfortaa',			fontGroups.decorative,	external, fontFamily.cursive],
			['Didact Gothic',		fontGroups.decorative,	external, fontFamily.sansSerif],
			['Marmelad',			fontGroups.decorative,	external, fontFamily.sansSerif],
			['Oranienbaum',			fontGroups.decorative,	external, fontFamily.serif],
			['Philosopher',			fontGroups.decorative,	external, fontFamily.sansSerif],

			['Lobster',				fontGroups.handwritten,	external, fontFamily.cursive],
			['Marck Script',		fontGroups.handwritten,	external, fontFamily.cursive],
			['Neucha',				fontGroups.handwritten,	external, fontFamily.cursive]
		],
		chineseFonts = [
			['Arial Unicode MS',	fontGroups.sansSerif,	internal, fontFamily.sansSerif],
			['SimSun',				fontGroups.sansSerif,	internal, fontFamily.serif],
			['SimSun Black',		fontGroups.sansSerif,	internal, fontFamily.serif],
			['NSimSun',				fontGroups.sansSerif,	internal, fontFamily.sansSerif],
			['SimHei',				fontGroups.sansSerif,	internal, fontFamily.sansSerif],
			['MingLiU',				fontGroups.sansSerif,	internal, fontFamily.serif],
			['PMingLiU',			fontGroups.sansSerif,	internal, fontFamily.serif],
			['Microsoft YaHei',		fontGroups.sansSerif,	internal, fontFamily.sansSerif]
		],
		groupingStoreConfig = {
			reader: new Ext.data.ArrayReader({
				fields: ['value', 'group', 'type', 'fontFamilyCss']
			}),
			groupField: 'group'
		};

	Ext.each([defaultFonts, latinFonts, cyrillicFonts, chineseFonts], function (fontList) {
		Ext.each(fontList, function (fontData) {
			var fontFamilyCss,
				displayName = fontData[0],
				fontFamily = fontData[3];

			fontFamilyCss = displayName.split(' ').length > 1
				? "'" + displayName + "', " +  fontFamily
				: displayName + ', ' + fontFamily;
			fontData[3] = fontFamilyCss;
		});
	});

	Ext.iterate(fontGroups, function (key, value) {
		fontFamilies[value] = Sb.Locale.get('fontFamily' + key.toUpperCaseFirst());
	});

	if (!Sb.Config.googleFontsEnabled) {
		latinFonts = latinFonts.filter(externalFontsExclude);
		cyrillicFonts = cyrillicFonts.filter(externalFontsExclude);
		chineseFonts = chineseFonts.filter(externalFontsExclude);
	}

	return {
		singleton: true,

		fontGroups: fontGroups,
		fontFamilies: fontFamilies,
		basicFonts: new Ext.data.ArrayStore({fields: ['value'], data: defaultFonts}),
		latinFonts: new Ext.data.GroupingStore(
			Ext.apply({data: defaultFonts.concat(latinFonts, cyrillicFonts)}, groupingStoreConfig)
		),
		cyrillicFonts: new Ext.data.GroupingStore(
			Ext.apply({data: defaultFonts.concat(cyrillicFonts)}, groupingStoreConfig)
		),
		chineseFonts: new Ext.data.GroupingStore(
			Ext.apply({data: defaultFonts.concat(chineseFonts)}, groupingStoreConfig)
		),

		/**
		 * @param {String} fontName
		 * @return {String} external font url
		 */
		getFontUrl: function (fontName) {
			return Sb.Editor.model.get('template').externalFonts[fontName]
				? Sb.Editor.model.get('template').externalFonts[fontName].url
				: '';
		},

		/**
		 * @param {String} fontFamily Example: "Arial" or "'Times New Roman', serif"
		 */
		loadFont: function (fontFamily) {
			var fontName = fontFamily.split(',')[0].replace(/('|")/g, ''),
				url = this.getFontUrl(fontName);

			if (loadedFonts[fontName] || !url) {
				return;
			}

			Sb.StyleManager.addStyleSheet('@import url(' + url + ');', 'fonts', fontName);
			loadedFonts[fontName] = true;
		}
	};
});
