/**
 * @class Sb.Highlighting
 * Highlighting placeholders when dragging module
 * @singleton
*/
Ext.define('Sb.Highlighting', {
	singleton: true,

	show: function () {
		if (!Ext.dd.DragDropMgr.dragCurrent.dragData) {
			return;
		}

		Ext.getBody().addClass('sb-highlight');

		Ext.ComponentMgr.all.each(function (component) {
			if (component === Ext.dd.DragDropMgr.dragCurrent.dragData.widget) {
				return;
			}

			if (component instanceof Sb.widget.Widget && component.model.get('visibilityMode') !== 'all') {
				if (!component.isVisible()) {
					component.show();
				}
				component.showVisibilityLabel();
			}
		});

		Sb.Editor.model.rootPage.widget.widgets.each(function (verticalContainer) {
			var pageContent;

			this.highlightSiteWideContainer(verticalContainer);

			pageContent = verticalContainer.queryBy(function (widget) {
				return widget.getType() == 'pagecontent';
			})[0];
			if (pageContent) {
				this.highlightPageWideContainer(pageContent);
			}
		}, this);

		this.headers = Ext.select('.sb-widget-header .widget-content');
		this.headers.each(function(el) {
			el.setStyle('min-height', el.getHeight() + 'px');
		});
	},

	highlightSiteWideContainer: function (container) {
		var containerCmp = Ext.getCmp(container.get('region'));

		if (containerCmp.isDroppable(Ext.dd.DragDropMgr.dragCurrent)) {
			this.highlightWidgets(container, 'sitewide');
		}
	},

	highlightPageWideContainer: function (pageContent) {
		var dragCurrent = Ext.dd.DragDropMgr.dragCurrent,
			pageContentCmp = Ext.getCmp('sb-widget-' + pageContent.get('uuid')),
			pageContentContainer = pageContentCmp.contentContainerWidget;

		if (pageContentContainer.isDroppable(dragCurrent)) {
			pageContentCmp.getEl().addClass('sb-highlight-pagecontent');
			this.highlightWidgets(pageContentContainer.model, 'pagewide');
		}
	},

	/**
	 * Highlight child widgets of rootWidget recursive
	 * @param {Sb.model.Widget} rootWidget
	 * @param {String} highlightType like 'sitewide' or 'pagewide'
	 */
	highlightWidgets: function (rootWidget, highlightType) {
		if (!rootWidget.widgets) {
			return;
		}

		rootWidget.widgets.each(function (widgetModel) {
			var widget;

			if ('sitewide' == highlightType && widgetModel.getType() == 'pagecontent') {
				return true;
			}

			if (widgetModel.widgets.length > 0) {
				this.highlightWidgets(widgetModel, highlightType);
			}
			widget = Ext.get('sb-widget-' + widgetModel.get('uuid'));
			if (widget) {
				widget.addClass('sb-highlight-' + highlightType);
			}

		}, this);
	},

	hide: function () {
		Ext.getBody().removeClass('sb-highlight');
		Ext.select('.sb-highlight-pagewide').removeClass('sb-highlight-pagewide');
		Ext.select('.sb-highlight-sitewide').removeClass('sb-highlight-sitewide');
		Ext.select('.sb-highlight-pagecontent').removeClass('sb-highlight-pagecontent');
		this.headers.setStyle('min-height', '');
		Ext.ComponentMgr.all.each(function (component) {
			if (component instanceof Sb.widget.Widget && component.model.get('visibilityMode') !== 'all') {
				component.updateVisibilityState();
				component.hideVisibilityLabel();
			}
		});
	}
});