/**
 * @class Sb.dashboard.model.Disqus
 * @extends Sb.dashboard.Model
 * Disqus comments tile model
 */
Ext.define('Sb.dashboard.model.Disqus', {
	extend: 'Sb.dashboard.Model',

	name: 'disqus',

	proxy: undefined,

	constructor: function (config) {
		this.addEvents('countnewchange');

		config = config || {};
		config.tasks = {
			loadData: {
				run: this.loadData.createDelegate(this),
				interval: 1000 * 60 * 10 // 10 minutes
			}
		};

		this.proxy = new Ext.data.PagingMemoryProxy({
			posts: [],
			total: 0,
			countNew: 0
		});

		Sb.dashboard.model.Disqus.superclass.constructor.call(this, config);
	},

	isEnabled: function () {
		return Sb.Config.widgets.indexOf('commenting') !== -1;
	},

	loadData: function () {
		if (Ext.util.JSON.encode(this.settings) == '{}') {
			this.settings.lastViewId = 0;
			this.on('settingsload', this.loadData, this, { single: true });
			this.loadSettings();
			return;
		}

		Sb.DisqusService.getRecentPosts(function (result) {
			if (result.success) {
				this.data = result.data;
				this.proxy.data = this.prepareData(this.data);
				this.fireEvent('datachanged');
			} else {
				this.lastError = result.message || Sb.Locale.get('unknownLoadingError');
				this.fireEvent('error', this, this.lastError, result.data.errorType);
			}
		}, this);
	},

	/**
	 * Convert disqus response to data store
	 * @param source
	 * @return {Object}
	 */
	prepareData: function (source) {
		var posts = [],
			linkToCommentLabel = Sb.Locale.get('linkToComment'),
			newPostsCount = 0,
			prevPosts = {},
			dateFormat = Sb.Config.locales[Sb.Config.locale].dateFormat + ' ' +
					Sb.Config.locales[Sb.Config.locale].timeFormat;

		if (this.proxy) {
			Ext.each(this.proxy.data.posts, function (post) {
				prevPosts[post.id] = post;
			});
		}

		Ext.iterate(source, function (row) {
			var post = {
				id: row.id,
				avatar: row.author.avatar.permalink,
				title: row.threadTitle,
				message: row.raw_message,
				threadLink: row.threadLink,
				linkToCommentLabel: linkToCommentLabel
			};

			var createdAt = Date.parseDate(row.createdAt, 'Y-m-dTH:i:s');

			if (prevPosts[row.id]) {
				post.newPost = prevPosts[row.id].newPost;
			} else {
				post.newPost = post.id > this.settings.lastViewId;
			}
			if (post.newPost) {
				newPostsCount++;
			}

			post.createdAt = createdAt.format(dateFormat);
			if (!row.author.isAnonymous) {
				post.username = row.author.name;
			} else {
				post.username = Sb.Locale.get('anonymousName');
			}

			posts.push(post);
		}, this);

		return {
			posts: posts,
			total: posts.length,
			countNew: newPostsCount
		};
	},

	setCountNew: function (count) {
		if (this.proxy.data.countNew != count) {
			this.proxy.data.countNew = count;
			this.fireEvent('countnewchange', this, count);
		}
	},

	getCountNew: function () {
		return this.proxy.data.countNew;
	}

});