/**
 * @class Sb.dashboard.tile.DisqusTile
 * @extends Sb.dashboard.Tile
 * Class for disqus recent comments tile
 * @xtype sb.dashboard.tile.disqus
 */
Ext.define('Sb.dashboard.tile.DisqusTile', {
	extend: 'Sb.dashboard.Tile',
	xtype: 'sb.dashboard.tile.disqus',

	initComponent: function () {
		this.model = this.manager.getModel('disqus');
		this.mon(this.model, 'datachanged', this.changeDataHandler, this);
		this.mon(this.model, 'countnewchange', this.changeDataHandler, this);
		this.mon(this.model, 'error', this.errorHandler, this);

		Ext.apply(this, {
			titleString: Sb.Locale.get('comments'),
			logoCls: 'sb-dashboard-logo-comments'
		});

		Sb.dashboard.tile.DisqusTile.superclass.initComponent.call(this);

		this.showMessage(Sb.Locale.get('loadComments'));
		this.model.startTask('loadData', true);
	},

	changeDataHandler: function () {
		if (!this.model.proxy.data.total) {
			this.showMessage(Sb.Locale.get('commentsNotFound'));
		} else {
			var el = this.getContentEl();

			if (!el.rendered) {
				return;
			}

			el.removeAll();
			el.add({
				xtype: 'box',
				html: Sb.Locale.get('countCommentsNew', this.model.getCountNew()) +
					' <a href="#">' + Sb.Locale.get(this.model.getCountNew() ? 'seeThemHere' : 'seeAllHere') + '</a>.'
			});
			el.doLayout();
			el.getEl().child('a').on('click', function () {
				var dashboard = Sb.Editor.windowManager.getManager(9).get('dashboardWindow'),
					notifyTab = dashboard.tabs.notificationTab;
				dashboard.tabs.setActiveTab(notifyTab);
				notifyTab.getComponent(0).setActiveTab(0);
			});
		}
	}
});