/**
 * @class Sb.dashboard.tile.GoogleAnalyticsDetailedTile
 * @extends Sb.dashboard.tile.GoogleAnalyticsTile
 * Class for Google analytics tile
 * @xtype sb.dashboard.tile.gadetailed
 */
Ext.define('Sb.dashboard.tile.GoogleAnalyticsDetailedTile', {
	extend: 'Sb.dashboard.tile.GoogleAnalyticsTile',
	xtype: 'sb.dashboard.tile.gadetailed',

	initComponent: function () {
		Ext.apply(this, {
			autoWidth: true,
			height: 620,
			layout: 'fit'
		});

		Sb.dashboard.tile.GoogleAnalyticsDetailedTile.superclass.initComponent.call(this);

		this.addClass('sb-dashboard-ga');
		this.titleBox.add({
			cls: 'sb-dashboard-ga-profiles',
			ref: '../../../profileSelectBox'
		});
	},

	dataChangeHandler: function () {
		if (!this.ownerCt.hidden) {
			this.activateHandler();
		} else {
			this.ownerCt.on('activate', this.activateHandler, this, { single: true });
		}
	},

	renderLinkToAuthorize: function () {
		this.profileSelectBox.hide();
		Sb.dashboard.tile.GoogleAnalyticsDetailedTile.superclass.renderLinkToAuthorize.call(this);
	},

	activateHandler: function () {
		// :TODO: 20120314 replace it on card layout
		if (this.model.data && this.model.isLogged) {
			this.profileSelectBox.show();
			var el = this.getContentEl();
			el.removeAll();
			el.add({
				height: 540,
				layout: {
					type: 'table',
					columns: 2
				},
				items: [{
						html: '<h1 class="sb-dashboard-ga-overview-title">' + Sb.Locale.get('gaOverview') + '</h1>',
						cellCls: 'sb-dashboard-without-bottom-padding',
						height: 30
					}, {
						xtype: 'combo',
						ctCls: 'sb-dashboard-ga-metrics',
						cellCls: 'sb-dashboard-without-bottom-padding',
						width: 250,
						mode: 'local',
						value: this.model.currentMainMetric.id,
						valueField: 'id',
						displayField: 'label',
						store: {
							xtype: 'jsonstore',
							data: this.model.metrics,
							fields: ['id', 'label']
						},
						listeners: {
							scope: this,
							select: function (combo, record, index) {
								this.model.currentMainMetric = this.model.metrics[index];
								this.showLoader();
								this.model.loadData();
							}
						}
					}, {
						colspan: 2,
						width: 677,
						items: {
						ref: '../../mainChartBox',
						listeners: {
							scope: this,
							afterrender: function () {
								var el = this.getContentEl();
								this.renderAreaChart(el.mainChartBox.getId(), this.model.data);
							}
						}
					}
				}, {
					ref: '../indicatorsBox',
					colspan: 2,
					cellCls: 'sb-dashboard-without-bottom-padding',
					layout: {
						type: 'table',
						columns: 2,
						tableAttrs: {
							cls: 'sb-dashboard-width100'
						}
					}
				}, {
					ref: '../trafficSourceBox',
					items: [{
						html: '<h1 class="sb-dashboard-ga-section-title">' + Sb.Locale.get('gaTrafficSourceOverview') + '</h1>'
					}, {
						listeners: {
							scope: this,
							afterrender: function (el) {
								if (this.model.extendedData.sourcesOverview) {
									this.renderPieChart(el.getId(), this.model.extendedData.sourcesOverview, {
										width: 340,
										height: 137
									});
								}
							}
						}
					}]
				}, {
					ref: '../geoBox',
					items: [{
						html: '<h1 class="sb-dashboard-ga-section-title">' + Sb.Locale.get('gaLocationVisitors') + '</h1>'
					}, {
						listeners: {
							scope: this,
							afterrender: function (el) {
								if (this.model.extendedData.geo) {
									this.renderGeoChart(el.getId(), this.model.extendedData.geo, {
										width: 340,
										height: 137
									});
								}
							}
						}
					}]
				}]
			});
			el.doLayout();

			this.renderProfilesSelect(this.profileSelectBox, this.profiles, this.model.get('tableId'));

			Ext.each(this.model.metrics, function (metric) {
				if (this.model.extendedData.indicators[metric.id]) {
					var data = this.model.extendedData.indicators[metric.id],
						child = new Ext.Container({
							cls: 'sb-dashboard-ga-indicator'
						}),
						value = data.value || 0;

					el.indicatorsBox.add(child);
					el.indicatorsBox.add({
						items: [child, {
							xtype: 'box',
							autoEl: {
								tag: 'span',
								html: '<span class="sb-dashboard-bold">' + value + '</span> ' +
									Sb.Locale.get(metric.indicatorLabel)
							},
							listeners: {
								scope: this,
								afterrender: function () {
									this.renderSmallAreaChart(child.getId(), data.series, {
										width: 70,
										height: 20
									});
								}
							}
						}]
					});
					el.indicatorsBox.doLayout();
				}
			}, this);
		}
	},

	renderPieChart: function (idElem, data, options) {
		var table = new google.visualization.DataTable();
		table.addColumn('string', '');
		table.addColumn('number', '');
		table.addRows(data);

		options = options || {};
		options = Ext.apply({
			width: '100%',
			height: '100%',
			backgroundColor: '#D3E9FF',
			chartArea: {
				left: 10,
				top: 10,
				height: '80%'
			}
		}, options);

		new google.visualization.PieChart(
			document.getElementById(idElem)
		).draw(table, options);
	},

	renderGeoChart: function (idElem, data, options) {
		var table = new google.visualization.DataTable();
		table.addColumn('string', '');
		table.addColumn('number', Sb.Locale.get('gaVisits'));
		table.addRows(data);

		options = options || {};
		options = Ext.apply({}, options);

		new google.visualization.GeoChart(
			document.getElementById(idElem)
		).draw(table, options);
	},

	renderSmallAreaChart: function (idElem, data, options) {
		var table = new google.visualization.DataTable();
		table.addColumn('string', Sb.Locale.get('gaDays'));
		table.addColumn('number', 'metric');
		table.addRows(data);

		options = options || {};
		options = Ext.apply({
			width: '100%',
			height: '100%',
			enableInteractivity: false,
			chartArea: {
				left: 0,
				top: 0,
				width: '100%',
				height: '95%'
			},
			legend: {
				position: 'none'
			},
			vAxis: {
				gridlines: {
					count: 0
				}
			},
			tooltip: {
				trigger: 'none'
			}
		}, options);

		var chart = new google.visualization.AreaChart(
			document.getElementById(idElem)
		);
		chart.draw(table, options);
	},

	showMessage: function (message, type) {
		Sb.dashboard.tile.GoogleAnalyticsDetailedTile.superclass.showMessage.call(this, message, type);
		this.getContentEl().getComponent(0).addClass('sb-dashboard-tile-message');
	}

});