/**
 * @class Sb.dashboard.tile.NotificationTile
 * @extends Sb.dashboard.Tile
 * Class for notification tiles
 */
Ext.define('Sb.dashboard.tile.NotificationTile', {
	extend: 'Sb.dashboard.Tile',

	width: '100%',

	proxy: undefined,

	itemsTpl: undefined,

	itemsPerPage: 5,

	initComponent: function () {
		this.proxy = new Ext.data.PagingMemoryProxy();

		this.store = this.createStore();

		var loader = {
			xtype: 'box',
			cls: 'sb-dashboard-message sb-dashboard-tile-message sb-dashboard-loader',
			isLoader: true,
			html: Sb.Locale.get('dataLoading') + ' <img class="sb-dashboard-loading-icon" src="' + Sb.Config.skinUrl+ '/images/loading.gif" alt="" />'
		};

		Ext.apply(this, {
			height: Ext.isWebKit ? 520 : 555,
			layout: 'fit',
			items: loader,
			autoWidth: true,
			buttonAlign: 'center',
			fbar: [{
				xtype: 'paging',
				ref: 'paging',
				store: this.store,
				pageSize: this.itemsPerPage,
				listeners: {
					scope: this,
					afterrender: function (pageTb) {
						pageTb.refresh.hide();
					},
					change: this.pagingChangeHandler
				}
			}]
		});

		Sb.dashboard.tile.NotificationTile.superclass.initComponent.call(this);

		this.mon(this.model, 'error', this.errorHandler, this);
		this.mon(this.model, 'datachanged', this.loadDataHandler, this);
		this.model.startTask('loadData');
	},

	pagingChangeHandler: Ext.emptyFn,

	doRefresh: function () {
		if (this.items.first().isLoader) {
			this.removeAll();
			this.add(new Ext.DataView({
				tpl: this.itemsTpl,
				cls: 'sb-notifications-dv',
				store: this.store,
				autoScroll: true
			}));
			this.doLayout();
		}
		this.fbar.paging.doRefresh();
	},

	getContentEl: function () {
		return this;
	},

	showMessage: function (message, type) {
		type = type || 'text';
		var el = this.getContentEl();
		el.removeAll();
		el.add({
			xtype: 'box',
			cls: 'sb-dashboard-message sb-dashboard-tile-message sb-dashboard-' + type,
			html: '<i></i><span>' + message + '</span>'
		});
		el.doLayout();
	}
});