/**
 * @class Sb.editor.CornersPanel
 * @extends Ext.Panel
 * Corners panel
 * @constructor
 * @param {Object} config The config object
 * @xtype cornerspanel
 */
Ext.define('Sb.editor.CornersPanel', {
	extend: 'Ext.Panel',
	xtype: 'cornerspanel',

	initComponent: function () {
		var column1 = {
			columnWidth: 0.5,
			items: [{
				xtype: 'containercombobox',
				ref: '../selectedContainer',
				anchor: '95%',
				listeners: {
					scope: this,
					select: this.onContainerSelect
				}
			}, {
				xtype: 'box',
				html: Sb.Locale.get('cornersContainerHelp')
			}]
		};

		var column2 = {
			columnWidth: 0.5,
			items: {
				xtype: 'checkboxgroup',
				ref: '../checkboxGroup',
				cls: 'sb-editor-corners-group',
				fieldLabel: Sb.Locale.get('cornersSelectRounded'),
				columns: 2,
				listeners: {
					scope: this,
					change: this.checkboxHandler
				},
				items: [{
					boxLabel: Sb.Locale.get('cornersRounded'),
					name: 'top-left'
				}, {
					boxLabel: Sb.Locale.get('cornersRounded'),
					name: 'top-right',
					ctCls: 'sb-checkbox-right'
				}, {
					boxLabel: Sb.Locale.get('cornersRounded'),
					name: 'bottom-left'
				}, {
					boxLabel: Sb.Locale.get('cornersRounded'),
					name: 'bottom-right',
					ctCls: 'sb-checkbox-right'
				}]
			}
		};

		Ext.apply(this, {
			title: Sb.Locale.get('cornersTabTitle'),
			cls: 'sb-editor-style-corners-panel',
			defaults: {
				border: false,
				layout: 'form',
				labelAlign: 'top'
			},
			helperMessage: Sb.Locale.get('cornersTabDescription'),
			layout: 'column',
			items: [column1, column2],
			listeners: {
				scope: this,
				activate: this.onActivate
			}
		});
		Sb.editor.CornersPanel.superclass.initComponent.apply(this, arguments);
	},

	/**
	 * Activate values and states of elements for corners
	 */
	onActivate: function () {
		this.onContainerSelect(this.selectedContainer);
	},

	/**
	 * Set border round to some container
	 * @param {Ext.form.CheckboxGroup} checkboxGroup
	 * @param {Array} checked
	 */
	checkboxHandler: function (checkboxGroup, checked) {
		var container = Sb.Editor.model.isRtl()
				? Sb.Utils.swapLeftAndRight(this.selectedContainer.getValue())
				: this.selectedContainer.getValue(),
			cssConfig = {'top-left': 0, 'top-right': 0, 'bottom-left': 0, 'bottom-right': 0};
		Ext.each(checked, function (checkbox) {
			cssConfig[checkbox.name] = '10px';
		});
		var cssValue = [cssConfig['top-left'], cssConfig['top-right'],
			cssConfig['bottom-right'], cssConfig['bottom-left']].join(' ');

		this.updatePreview(checkboxGroup, cssValue);
		if (Sb.Editor.model.isRtl()) {
			cssValue = Sb.Utils.swapCssCornersValues(cssValue);
		}
		Sb.Editor.model.design.setDecoration('borderRadius', container + 'BorderRadius', cssValue);
		Sb.editor.UiContext.registerChanges();
	},

	/**
	 * @param {Sb.form.ContainerComboBox} combo
	 */
	onContainerSelect: function (combo) {
		var place = Sb.Editor.model.isRtl() ? Sb.Utils.swapLeftAndRight(combo.getValue()) : combo.getValue(),
			cssValue = Sb.Editor.model.design.getDecoration('borderRadius', place + 'BorderRadius'),
			values = Sb.Editor.model.isRtl()
				? Sb.Utils.swapCssCornersValues(cssValue).split(' ')
				: cssValue.split(' '),
			notNull = function (item) {
				return item && item != '0';
			},
			config = {'top-left': notNull(values[0]), 'top-right': notNull(values[1]),
			'bottom-right': notNull(values[2]), 'bottom-left': notNull(values[3])};
		this.checkboxGroup.eachItem(function (checkbox) {
			checkbox.setRawValue(config[checkbox.name]);
		});
		this.updatePreview(this.checkboxGroup, cssValue);
	},

	/**
	 * @param {Ext.form.CheckboxGroup} checkboxGroup
	 * @param {String}
	 */
	updatePreview: function (checkboxGroup, cssValue) {
		checkboxGroup.getEl().setStyle({
			'-webkit-border-radius': cssValue,
			'-moz-border-radius': cssValue,
			'border-radius': cssValue
		});
	}
});