/**
 * @class Sb.editor.MainWindow.FakeProgressBar
 * @extends Ext.ProgressBar
 * Class for publish progressbar
 * @constructor
 * @param {Object} config The config object
 * @xtype fakeprogressbar
 */
Ext.define('Sb.editor.MainWindow.FakeProgressBar', {
	extend: 'Ext.ProgressBar',
	xtype: 'fakeprogressbar',

	/*
	 * @param {Object} [config] Configuration options
	 * @return {Sb.editor.MainWindow.FakeProgressBar} this
	 */
	wait: function (o) {
		if (!this.waitTimer) {
			var scope = this;
			o = o || {};
			this.updateText(Sb.Locale.get(o.localeKey, 0));
			this.waitTimer = Ext.TaskMgr.start({
				run: function (i) {
					var progress;
					var time = new Date().getTime() - this.waitTimer.taskStartTime;
					if (time < o.duration / 9) {
						progress = (9 * time) / (10 * o.duration);
					} else {
						progress = 1 - 6 * Math.sqrt(o.duration * time) / (65 * time - 5 * o.duration);
					}

					this.updateProgress(progress, Sb.Locale.get(o.localeKey, Math.ceil(progress * 100)), o.animate);
				},
				interval: o.interval || 100,
				scope: scope
			});
		}
		return this;
	},

	gotoEnd: function (o) {
		if (this.waitTimer) {
			Ext.TaskMgr.stop(this.waitTimer);
		}
		o = o || {};
		this.waitTimer = Ext.TaskMgr.start({
			run: function (i) {
				var time = new Date().getTime() - this.waitTimer.taskStartTime;
				var progress = this.value + Math.pow((time - (1 - this.value) * 1000) / 1000, 2);
				this.updateProgress(progress, Sb.Locale.get(o.localeKey, Math.ceil(progress * 100)));
				if (progress > 1) {
					Ext.TaskMgr.stop(this.waitTimer);
				}
			},
			interval: 50,
			duration: 1000,
			scope: this,
			onStop: function () {
				if (o.fn) {
					o.fn.apply(o.scope || this);
				}
			}
		});
	}
});