/**
 * @class Sb.editor.LayoutDoll
 * @extends Ext.BoxComponent
 * Control for changing lyout
 * @constructor
 * @param {Object} config The config object
 * @xtype layoutdoll
 */
Ext.define('Sb.editor.LayoutDoll', {
	extend: 'Ext.BoxComponent',
	xtype: 'layoutdoll',

	constructor: function (config) {
		config = Ext.apply({
			headerLayout: 'watermark',
			contentLayout: 'watermark',
			footerLayout: 'watermark',
			sidebarFirstVisible: true,
			sidebarSecondVisible: true,
			sidebarsSwitched: false
		}, config);

		Sb.editor.LayoutDoll.superclass.constructor.call(this, config);
	},

	onRender: function () {
		var dollLayout =
			'<div class="sb-doll-header">' +
				'<div class="sb-doll-header-container sb-doll-container"><table><tr><td></td></tr></table></div>' +
			'</div>' +
			'<div class="sb-doll-columns">' +
				'<div class="sb-doll-sidebar-first sb-doll-container"><table><tr><td></td></tr></table></div>' +
				'<div class="sb-doll-sidebar-second sb-doll-container"><table><tr><td></td></tr></table></div>' +
				'<div class="sb-doll-content-container sb-doll-container"><table><tr><td></td></tr></table></div>' +
			'</div>' +
			'<div class="sb-doll-footer">' +
				'<div class="sb-doll-footer-container sb-doll-container"><table><tr><td></td></tr></table></div>' +
			'</div>';

		this.autoEl = {
			html: dollLayout,
			cls: 'sb-doll'
		};

		Sb.editor.LayoutDoll.superclass.onRender.apply(this, arguments);

		this.header = this.el.first();
		this.headerContainer = this.header.first();
		this.headerContent = this.headerContainer.child('td');
		this.columns = this.header.next();
		this.sidebarFirst = this.columns.first().setVisibilityMode(Ext.Element.DISPLAY);
		this.sidebarFirstContent = this.sidebarFirst.child('td');
		this.sidebarSecond = this.sidebarFirst.next().setVisibilityMode(Ext.Element.DISPLAY);
		this.sidebarSecondContent = this.sidebarSecond.child('td');
		this.contentContainer = this.sidebarSecond.next();
		this.content = this.contentContainer.child('td');
		this.footer = this.columns.next();
		this.footerContainer = this.footer.first();
		this.footerContent = this.footerContainer.child('td');

		Ext.each([
			'headerContainer',
			'sidebarFirst',
			'sidebarSecond',
			'contentContainer',
			'footerContainer'
		], function (name) {
			var param = name.replace('Container', '');
			this[name].on('click', this.onClick.createDelegate(this, [param]));
			this[name].on('mouseenter', this.onMouseEnter.createDelegate(this, [param]));
			this[name].on('mouseleave', this.updateText.createDelegate(this, [param]));

			this.updateText(param);
		}, this);

		this.refresh();
	},

	refresh: function () {
		Ext.each(['header', 'columns', 'footer'], function(container) {
			this[container].removeClass([
				'sb-doll-layout-watermark',
				'sb-doll-layout-elastic',
				'sb-doll-layout-elastic-background',
				'sb-doll-columns-hide-left',
				'sb-doll-columns-hide-right'
			]);

			if (container == 'columns') {
				this.columns.addClass('sb-doll-layout-' + this.contentLayout);
			} else {
				this[container].addClass('sb-doll-layout-' + this[container + 'Layout']);
			}
		}, this);

		this.sidebarFirst.setStyle('float', this.sidebarsSwitched ? 'right' : 'left');
		this.sidebarSecond.setStyle('float', this.sidebarsSwitched ? 'left' : 'right');

		this.sidebarFirst[(this.sidebarFirstVisible) ? 'show' : 'hide']();
		this.sidebarSecond[(this.sidebarSecondVisible) ? 'show' : 'hide']();

		if (!this.sidebarFirstVisible) {
			this.columns.addClass('sb-doll-columns-hide-' + (this.sidebarsSwitched ? 'right' : 'left'));
		}
		if (!this.sidebarSecondVisible) {
			this.columns.addClass('sb-doll-columns-hide-' + (this.sidebarsSwitched ? 'left' : 'right'));
		}
	},

	onClick: function (container) {
		var currentLayout = this[container + 'Layout'];

		if (container == 'header' || container == 'footer') {
			if (currentLayout == 'watermark') {
				this[container + 'Layout'] = 'elastic-background';
			} else if (currentLayout == 'elastic-background') {
				this[container + 'Layout'] = 'elastic';
			} else {
				this[container + 'Layout'] = 'watermark';
			}
		} else if (container == 'content') {
			this[container + 'Layout'] = (currentLayout == 'watermark') ? 'elastic-background' : 'watermark';
		}

		this.refresh();
		this.onMouseEnter(container);
		this.fireEvent('change', this.get());
	},

	onMouseEnter: function (container) {
		var currentLayout = this[container + 'Layout'],
			locale = '';

		if (container == 'header' || container == 'footer') {
			if (currentLayout == 'watermark') {
				locale = 'layoutContainerToElasticBackground';
			} else if (currentLayout == 'elastic-background') {
				locale = 'layoutContainerToElastic';
			} else {
				locale = 'layoutContainerToWatermark';
			}
			this[container + 'Content'].update(Sb.Locale.get(locale));
		} else if (container == 'content') {
			locale = currentLayout == 'watermark' ? 'layoutContainerToElasticBackground' : 'layoutContainerToWatermark';
			this.content.update(Sb.Locale.get(locale));
		}
	},

	updateText: function (container) {
		if (container == 'header' || container == 'footer') {
			this[container + 'Content'].update(Sb.Locale.get('layout' + Ext.util.Format.ucfirst(container)));
		} else if (container == 'sidebarFirst' || container == 'sidebarSecond') {
			this[container + 'Content'].update(Sb.Locale.get(
				container == 'sidebarFirst' ? 'layoutSidebarFirst' : 'layoutSidebarSecond'
			));
		} else if (container == 'content') {
			this.content.update(Sb.Locale.get('layoutContent'));
		}
	},

	get: function () {
		return {
			headerLayout: this.headerLayout,
			contentLayout: this.contentLayout,
			footerLayout: this.footerLayout
		};
	},

	set: function (config) {
		if (Sb.Editor.model.isRtl()) {
			var origConfig = Ext.apply({}, config);
			config.sidebarFirstVisible = origConfig.sidebarSecondVisible;
			config.sidebarSecondVisible = origConfig.sidebarFirstVisible;
		}
		Ext.apply(this, config);
		this.refresh();
	}
});