/**
 * @class Sb.editor.MobileSimulator
 * @extends Sb.Window
 * Mobile simulator window
 * @constructor
 * @param {Object} config The config object
 */
Ext.define('Sb.editor.MobileSimulator', {
	extend: 'Sb.Window',

	id: 'mobileSimulator',
	constrainHeader: true,

	devices: [{
		text: 'iPhone 4',
		cssClass: 'iphone4',
		width:  231,
		height: 454
	}, {
		text: 'Galaxy S II',
		cssClass: 'galaxys2',
		width:  260,
		height: 493
	}, {
		text: 'HTC Sensation 4G',
		cssClass: 'htcsensation4g',
		width:  257,
		height: 496
	}, {
		text: 'Galaxy Ace',
		cssClass: 'galaxyace',
		width:  236,
		height: 443
	}, {
		text: 'iPhone 5',
		checked: true,
		cssClass: 'iphone5',
		width:  231,
		height: 487
	}],

	orientation: 'vertical',

	initComponent: function () {
		Ext.apply(this, {
			title: Sb.Locale.get('mobileSimulatorTitle'),
			baseCls: 'sb-dialog',
			resizable: false,
			modal: true,
			closeAction: 'hide',
			shadow: false,
			items: {
				xtype: 'component',
				autoEl: {
					id: 'mobileSimulatorIframe',
					tag: 'iframe',
					scrolling: 'no'
				},
				listeners: {
					afterrender: function () {
						var iframeWindow = this.getEl().dom.contentWindow;

						this.getEl().on('load', function () {
							var $ = iframeWindow.siteBuilderJs,
								$iframeWindow = $(iframeWindow),
								clickX,
								clickY,
								drag = false,
								mouseScroll = function (mouseData) {
									var delta = mouseData > 0 ? - 200 : 200;
									$iframeWindow.scrollTop($iframeWindow.scrollTop() + delta);
								};

							$('body').addClass('mobile-simulator');
							if (Sb.Editor.model.get('mobileOptimizationsEnabled')) {
								iframeWindow.showMobileSwitcher($);
							}

							$(iframeWindow.document).on({
								mousemove: function (e) {
									e.preventDefault();

									var mouseLeftKey = 1;
									if (drag && e.which == mouseLeftKey) {
										$iframeWindow.scrollLeft($iframeWindow.scrollLeft() + (clickX - e.pageX));
										$iframeWindow.scrollTop($iframeWindow.scrollTop() + (clickY - e.pageY));
									}
								},
								mousedown: function (e) {
									e.preventDefault();
									clickX = e.pageX;
									clickY = e.pageY;
									drag = true;
								},
								mouseup: function (e) {
									drag = false;
								},
								mousewheel: function (e) {
									mouseScroll(e.originalEvent.wheelDelta);
								},
								// mousewheel for Firefox
								DOMMouseScroll: function (e) {
									mouseScroll(-e.originalEvent.detail);
								}
							});
						});
					}
				}
			},
			buttonAlign: 'center',
			fbar: [{
				text: Sb.Locale.get('mobileSimulatorSwitch'),
				menu: new Ext.menu.Menu({
					items: this.devices,
					defaults: {
						checked: false,
						group: 'mobileSimulatorDevice'
					},
					listeners: {
						scope: this,
						click: this.showDevice
					}
				})
			}, {
				text: Sb.Locale.get('mobileSimulatorRotate'),
				scope: this,
				handler: function () {
					this.orientation = this.orientation == 'vertical' ? 'horizontal' : 'vertical';
					this.showDevice();
				}
			}]
		});

		this.callParent();
	},

	onShow: function () {
		var iframe = Ext.get('mobileSimulatorIframe'),
			date = new Date();

		date.setDate(date.getDate() + 365);
		document.cookie = 'deviceView=mobile; path=/; expires=' + date.toUTCString();

		this.showDevice();
		iframe.set({
			src: Sb.Utils.antiCache(Sb.Editor.model.getUrl() + '/__preview', true)
		});
	},

	onHide: function () {
		document.cookie = 'deviceView=; path=/; expires=Thu, 01 Jan 1970 00:00:01 GMT;';
	},

	showDevice: function () {
		var device = Ext.menu.MenuMgr.getCheckedItem('mobileSimulatorDevice'),
			cls = device.cssClass,
			horizontalPadding = 20,
			verticalPadding = 86,
			width = device.width,
			height = device.height;

		this.body.dom.className = this.baseCls + '-body';

		if (this.orientation == 'horizontal') {
			cls += '-horizontal';
			height = device.width;
			width = device.height;
		}

		this.setSize(width + horizontalPadding, height + verticalPadding);
		this.body.addClass(cls);
	}
});

