/**
 * @class Sb.editor.SettingsPanel
 * @extends Sb.editor.BaseFormPanel
 * Settings panel
 * @constructor
 * @param {Object} config The config object
 * @xtype settingspanel
 */
Ext.define('Sb.editor.SettingsPanel', {
	extend: 'Sb.editor.BaseFormPanel',
	xtype: 'settingspanel',

	faviconFile: null,

	// private
	initComponent: function () {
		var localesData = [],
			tabs = [];
		this.model = Sb.Editor.model;

		Ext.iterate(Sb.Config.locales, function (code, data) {
			localesData.push([
				code,
				data.name,
				'background-image: url(' + Sb.Config.siteResourcesUrl + '/images/flags/16/' +
					code.split('_')[1].toLowerCase() + '.png);',
				data.hasTranslation
			]);
		});

		new Ext.data.GroupingStore({
			id: 'locales',
			reader: new Ext.data.ArrayReader({
				fields: [
					'code',
					'name',
					'iconStyle',
					{name: 'hasTranslation', type: 'boolean'}
				]
			}),
			data: localesData,
			groupField: 'hasTranslation',
			groupDir: 'DESC',
			sortInfo: {
				field: 'name'
			}
		});

		tabs.push({
			itemId: 'siteInfo',
			title: Sb.Locale.get('siteInfoTabTitle'),
			cls: 'sb-editor-settings-siteinfo-panel',
			autoHeight: true,
			helperMessage: Sb.Locale.get('siteInfoTabHelperMsg'),
			layout: 'form',
			listeners: {
				scope: this,
				activate: function () {
					this.updateLngButtons();
					this.changeLocale(Sb.Editor.siteView.getCurrentSiteLocale());

					//:INFO: workaround for bug #49418
					Ext.getCmp('settingsPanelSiteDescription').syncSize();
					Ext.getCmp('settingsPanelMetaKeywords').syncSize();
				}
			},
			items: [{
				xtype: 'languagebuttons',
				ref: '../../lngButtons',
				value: Sb.Editor.siteView.getCurrentSiteLocale(),
				data: Sb.Editor.model.get('locales'),
				listeners: {
					scope: this,
					selectionchange: function (cmp) {
						this.changeLocale(cmp.getValue());
					}
				}
			}, {
				xtype: 'fieldgroup',
				cls: 'sb-editor-box',
				items: [{
					xtype		: 'hidden',
					name		: 'customHead'
				}, {
					xtype		: 'hidden',
					name		: 'googleAnalyticsCode'
				}, {
					xtype		: 'hidden',
					name		: 'cookieBlockerCode'
				}, {
					xtype		: 'hidden',
					name		: 'robotsTxt'
				}, {
					xtype		: 'checkbox',
					name		: 'cookiePolicyWidgetEnabled',
					hidden		: true
				}, {
					anchor		: '100%',
					xtype		: 'textfield',
					fieldLabel	: Sb.Locale.get('siteName'),
					name		: 'companyName',
					maxLength	: 255,
					enableKeyEvents: true,
					listeners: {
						scope: this,
						keyup: this.checkChanges
					}
				}, {
					xtype		: 'checkbox',
					hideLabel	: true,
					boxLabel	: Sb.Locale.get('useSiteNameInPages'),
					name		: 'pagesUseName',
					listeners: {
						check: function () {
							Sb.editor.UiContext.registerChanges();
						}
					}
				}, {
					xtype		: 'box',
					cls			: 'sb-editor-box-description',
					html		: Sb.Locale.get('siteTitleDescription')
				}]
			}, {
				xtype: 'fieldgroup',
				cls: 'sb-editor-box',
				items: [{
					anchor			: '100%',
					xtype			: 'textarea',
					style			: {'overflow-x' : 'hidden'},
					id				: 'settingsPanelSiteDescription',
					fieldLabel		: Sb.Locale.get('siteDescription'),
					height			: 45,
					maxLength		: 255,
					maxLengthText	: Sb.Locale.get('siteDescriptionMaxLengthExceeded'),
					msgTarget		: 'under',
					name			: 'description',
					enableKeyEvents: true,
					listeners: {
						scope: this,
						keyup: this.checkChanges
					}
				}, {
					xtype		: 'box',
					cls			: 'sb-editor-box-description',
					html		: Sb.Locale.get('siteDescriptionDesc')
				}]
			}, {
				xtype: 'fieldgroup',
				cls: 'sb-editor-box',
				items: [{
					anchor			: '100%',
					xtype			: 'textarea',
					id				: 'settingsPanelMetaKeywords',
					style			: {'overflow-x' : 'hidden'},
					fieldLabel		: Sb.Locale.get('metaKeywords'),
					height			: 45,
					maxLength		: 255,
					maxLengthText	: Sb.Locale.get('metaKeywordsMaxLengthExceeded'),
					msgTarget		: 'under',
					name			: 'keywords',
					enableKeyEvents: true,
					listeners: {
						scope: this,
						keyup: this.checkChanges
					}
				}, {
					xtype		: 'box',
					cls			: 'sb-editor-box-description',
					html		: Sb.Locale.get('metaKeywordsDesc')
				}]
			}, {
				layout: 'column',
				cls: 'sb-editor-box sb-editor-box-last',
				border: false,
				defaults: {border: false},
				items: [{
					xtype: 'fieldgroup',
					width: 70,
					cls: 'sb-editor-settings-icon',
					items: {
						xtype		: 'displayfield',
						fieldLabel	: Sb.Locale.get('faviconTitle'),
						id			: 'sb-favicon-preview'
					}
				}, {
					columnWidth: 0.99,
					autoEl: {tag: 'form'},
					layout: 'form',
					labelAlign: 'top',
					id: 'sb-favicon-form',
					items: {
						id			: 'sb-favicon-upload',
						xtype		: 'fileuploadfield',
						regex		: /(\.ico$)/i,
						msgTarget	: 'under',
						anchor		: '100%',
						buttonText	: Sb.Locale.get('browseButtonText'),
						fieldLabel	: Sb.Locale.get('uploadFavicon'),
						emptyText	: Sb.Locale.get('clickToBrowseAndUploadFile'),
						name		: 'faviconfile',
						listeners	: {
							scope: this,
							fileselected: this.onFaviconSelected
						}
					}
				}, {
					xtype	: 'button',
					text	: Sb.Locale.get('Remove'),
					iconCls	: 'sb-delete-icon',
					cls		: 'sb-editor-settings-remove-btn',
					scope	: this,
					handler	: function () {
						Sb.editor.UiContext.registerChanges();
						this.faviconFile = 'remove';
						Ext.getDom('sb-favicon-preview').innerHTML = '';
						Ext.getCmp('sb-favicon-upload').reset();
					}
				}]
			}, {
				xtype	: 'box',
				cls		: 'sb-editor-box-description',
				html	: Sb.Locale.get('faviconDesc')
			}]
		});

		tabs.push({
			id: 'sb-settings-panel-language-tab',
			itemId: 'language',
			title: Sb.Locale.get('languageTabTitle'),
			cls: 'sb-editor-settings-language-panel sb-editor-box sb-editor-box-last',
			autoHeight: true,
			xtype: 'form',
			ref: '../languageTab',
			labelAlign : 'top',
			items: [{
				xtype: 'compositefield',
				fieldLabel: Sb.Locale.get('siteLanguage'),
				ref: 'mainLanguageField',
				isFirstLngField: true,
				name: 'locale',
				getValue: function () {
					return this.comboField.getValue();
				},
				items: [this.getLanguageComboCfg(), {
					xtype: 'label',
					style: {
						lineHeight: '24px'
					},
					flex: 1,
					html: Sb.Locale.get('siteMainLanguage'),
					cls: 'sb-editor-box-description',
					hidden: true,
					ref: 'labelBox'
				}]
			}, {
				xtype: 'container',
				layout: 'hbox',
				layoutConfig: {
					padding: '0 18 5 0'
				},
				items: [{
					flex: 3,
					xtype: 'label'
				}, {
					flex: 1,
					xtype: 'button',
					iconCls: 'sb-add-icon',
					cls: 'sb-add-locale-button',
					id: 'addLocaleButton',
					text: Sb.Locale.get('siteLanguageAddButtonTitle'),
					height: 24,
					hidden: !Sb.Config.isEnabledMlw,
					scope: this,
					handler: function () {
						Sb.editor.UiContext.registerChanges();
						this.addLanguageField();
						this.updateAddLocaleButton();
					}
				}]
			}, {
				xtype: 'label',
				autoHeight: true,
				html: Sb.Locale.get('siteLanguageDescription') +
					(Sb.Config.isEnabledMlw ? '<br>' + Sb.Locale.get('siteMultiLanguageDescription') : ''),
				cls: 'sb-editor-box-description'
			}]
		});

		// It will be component or design element in future.
		var boxCls = 'sb-editor-box',
			headTpl = '<div class="{0} {5}"><div class="{0}-title"><a href="{4}" class="{1}">{2}</a></div>' +
				'<div class="{0}-description">{3}</div></div>',
			seoTabLinksList,
			seoTabDescription,
			seoTab;

		seoTabDescription = '<p>' + Sb.Locale.get('seoTabDescription') + '</p>';
		seoTabLinksList = String.format(
				headTpl, boxCls, 'sb-ownership-link',
				Sb.Locale.get('seoOwnershipTitle'), Sb.Locale.get('seoOwnershipDesc'),
				'#', ''
			) +
			String.format(
				headTpl, boxCls, 'sb-edit-head-link',
				Sb.Locale.get('seoEditHeadTitle'), Sb.Locale.get('seoEditHeadDesc'),
				'#', ''
			);

		if (Sb.Config.googleAnalyticsEnabled) {
			seoTabLinksList += String.format(
				headTpl, boxCls, 'sb-google-analitics-link',
				Sb.Locale.get('seoGoogleAnTitle'), Sb.Locale.get('seoGoogleAnDesc'),
				'#', ''
			);
		}
		// seoTabLinksList += String.format(
		// 	headTpl, boxCls, 'sb-cookie-blocker-link',
		// 	Sb.Locale.get('seoCookieBlockerTitle'), Sb.Locale.get('seoCookieBlockerDesc'),
		// 	'#', ''
		// );

		seoTabLinksList += String.format(
				headTpl, boxCls, 'sb-download-sitemap-link',
				Sb.Locale.get('seoDownloadSitemapTitle'), Sb.Locale.get('seoDownloadSitemapDesc'),
				Sb.Config.baseUrl + '/Download/SiteMap?X-CSRF-Token=' + Sb.Config.csrfToken, ''
			) +
			String.format(
				headTpl, boxCls, 'sb-edit-robots-txt-link',
				Sb.Locale.get('seoEditRobotsTxtTitle'), Sb.Locale.get('seoEditRobotsTxtDesc'),
				'#', ''
			) +
			String.format(
				headTpl, boxCls, 'sb-cookie-policy-widget-link',
				Sb.Locale.get('seoCookieBlockerTitle'), Sb.Locale.get('seoCookieBlockerDesc'),
				'#', 'sb-editor-box-last'
			);

		tabs.push({
			id: 'sb-settings-panel-seo-tab',
			itemId: 'seo',
			title: Sb.Locale.get('seoTabTitle'),
			cls: 'sb-editor-settings-seo-panel',
			helperMessage: Sb.Locale.get('seoTabHelperMsg'),
			autoHeight: true,
			items: [{
				xtype: 'box',
				html: seoTabDescription
			}, {
				xtype: 'container',
				cls: boxCls,
				items: [{
					xtype: 'checkbox',
					name: 'mobileOptimizationsEnabled',
					boxLabel : Sb.Locale.get('mobileOptimizationsEnabled'),
					listeners: {
						check: function () {
							Sb.editor.UiContext.registerChanges();
						}
					}
				}, {
					xtype: 'box',
					cls: 'sb-editor-box-description',
					html: Sb.Locale.get('mobileOptimizationsDescription')
				}]
			}, {
				xtype: 'box',
				html: seoTabLinksList,
				listeners: {
					scope: this,
					afterrender: this.seoTabAfterRender
				}
			}]
		});

		var facebookIntegrationCheckbox;
		if (Sb.Config.general.facebookApplicationUrl) {
			facebookIntegrationCheckbox = new Ext.form.Checkbox({
				name: 'isFacebookIntegration',
				boxLabel: Sb.Locale.get('socialFacebookEnableIntegration'),
				handler: function (me, checked) {
					if (checked) {
						return;
					}
					Ext.Msg.show({
						title: Sb.Locale.get('confirmChangesDialogTitle'),
						msg: Sb.Locale.get('socialFacebookDisableIntegrationConfirmation', Sb.Config.general.product),
						closable: false,
						buttons: Ext.Msg.YESNO,
						fn: function (btn) {
							if (btn == 'no') {
								me.setValue(true);
							}
						}
					});
				}
			});

			tabs.push({
				id: 'sb-settings-panel-social-tab',
				itemId: 'social',
				title: Sb.Locale.get('socialTabTitle'),
				cls: 'sb-editor-settings-social-panel',
				html: '<p>' + Sb.Locale.get('socialTabDescription') + '</p>' +
					String.format(headTpl, boxCls, 'sb-facebook-link', Sb.Locale.get('socialFacebookTitle'),
						Sb.Locale.get('socialFacebookDescription', Sb.Config.general.product) +
							' <a href="' + Sb.Editor.getHelpUrl('/Panel/Settings/Facebook_integration') +
							'" target="_blank">' + Sb.Locale.get('learnMore') + '</a>',
						'#', 'sb-editor-box-last'),
				listeners: {
					afterrender: function (p) {
						var fbLink = p.el.child('.sb-facebook-link');
						fbLink.on('click', Sb.Editor.startFacebookIntegration);
						facebookIntegrationCheckbox.render(fbLink.parent('.' + boxCls));
					}
				}
			});
		}

		var siteOwnerInfo = Sb.Editor.model.get('ownerInfo');
		tabs.push({
			title: Sb.Locale.get('siteOwnerInfoTabTitle'),
			cls: 'sb-editor-settings-siteownerinfo-panel',
			autoHeight: true,
			layout: 'form',
			labelAlign: 'left',
			labelWidth: 175,
			defaults: {
				xtype: 'textfield',
				maxLength: 255,
				width: 280,
				msgTarget: 'under'
			},
			items: [{
				xtype: 'box',
				width: 'auto',
				html: '<p>' + Sb.Locale.get('siteOwnerInfoTabDescription') + '</p>'
			}, {
				fieldLabel: Sb.Locale.get('siteOwnerInfoPersonalName'),
				name: 'siteOwnerInfoPersonalName',
				value: siteOwnerInfo.personalName
			}, {
				fieldLabel: Sb.Locale.get('siteOwnerInfoCompanyName'),
				name: 'siteOwnerInfoCompanyName',
				value: siteOwnerInfo.companyName
			}, {
				fieldLabel: Sb.Locale.get('siteOwnerInfoPhone'),
				name: 'siteOwnerInfoPhone',
				value: siteOwnerInfo.phone
			}, {
				fieldLabel: Sb.Locale.get('siteOwnerInfoEmail'),
				name: 'siteOwnerInfoEmail',
				value: siteOwnerInfo.email,
				vtype: 'email',
				validationEvent : 'change'
			}, {
				xtype: 'textarea',
				height: 52,
				fieldLabel: Sb.Locale.get('siteOwnerInfoAddress'),
				name: 'siteOwnerInfoAddress',
				value: siteOwnerInfo.address
			}, {
				fieldLabel: Sb.Locale.get('siteOwnerInfoCity'),
				name: 'siteOwnerInfoCity',
				value: siteOwnerInfo.city
			}, {
				fieldLabel: Sb.Locale.get('siteOwnerInfoState'),
				name: 'siteOwnerInfoState',
				value: siteOwnerInfo.state
			}, {
				fieldLabel: Sb.Locale.get('siteOwnerInfoZip'),
				name: 'siteOwnerInfoZip',
				value: siteOwnerInfo.zip
			}, {
				fieldLabel: Sb.Locale.get('siteOwnerInfoCountry'),
				name: 'siteOwnerInfoCountry',
				value: siteOwnerInfo.country
			}]
		});

		Ext.apply(this, {
			cls: 'sb-editor-settings-panel',
			title: Sb.Locale.get('settingsTitle'),
			helperMessage: Sb.Locale.get('settingsDescription'),
			tabTip: Sb.Locale.get('settingsDescriptionOnHover'),
			autoHeight: true,
			items: {
				xtype: 'sbtabpanel',
				tabWidth: 90,
				tabPosition: 'left',
				activeTab: 0,
				border: false,
				cls: 'sb-editor-settings-panel-tabs',
				items: tabs
			},
			listeners: {
				scope: this,
				activate: this.onActivate,
				afterrender: function () {
					this.model.get('locales').forEach(function(locale) {
						if (locale !== this.model.get('locale')) {
							this.addLanguageField(locale);
						}
					}, this);
				}
			}
		});

		Sb.editor.SettingsPanel.superclass.initComponent.apply(this, arguments);
		this.bodyCfg.tag = 'div';

		if (facebookIntegrationCheckbox) {
			this.form.add(facebookIntegrationCheckbox);
		}

		/**
		 * Inner tab panel
		 * @type Ext.TabPanel
		 * @property tabs
		 */
		this.tabs = this.items.item(0);
	},

	updateLngButtons: function () {
		var locales = this.languageTab.rendered ? this.getLocalesFromForm() : this.model.get('locales');

		this.lngButtons
			.setData(locales)
			.setValue(Sb.Editor.siteView.getCurrentSiteLocale())
			.setVisible(locales.length > 1);
	},

	onActivate: function () {
		var pagesCheck = this.form.findField('pagesUseName');

		this.callParent();

		this.memento = this.model.saveToMemento();
		pagesCheck.suspendEvents();
		this.form.setValues({
			customHead: this.model.get('customHead'),
			googleAnalyticsCode: this.model.get('googleAnalyticsCode'),
			cookieBlockerCode: this.model.get('cookieBlockerCode'),
			robotsTxt: this.model.get('robotsTxt'),
			pagesUseName: this.model.get('pagesUseName'),
			isFacebookIntegration: this.model.get('isFacebookIntegration'),
			locale: this.model.get('locale'),
			cookiePolicyWidgetEnabled: this.model.get('cookiePolicyWidgetEnabled'),
			mobileOptimizationsEnabled: this.model.get('mobileOptimizationsEnabled')
		});
		pagesCheck.resumeEvents();
		this.faviconFile = this.model.get('faviconFile');
		if ('favicon.ico' === this.faviconFile) {
			Ext.getDom('sb-favicon-preview').innerHTML = '<img src="' + Sb.Utils.antiCache(this.model.getUrl() +
				'/favicon.png', true) + '">';
		}

		this.updateLngButtons();
		this.changeLocale(Sb.Editor.siteView.getCurrentSiteLocale());
		this.updateAddLocaleButton();
	},

	checkChanges: function (field) {
		this.model.set(field.name, field.getValue());
		Sb.editor.UiContext.registerChanges();
	},

	ownershipWinOpen: function () {
		var ownershipWin = new Sb.Window({
			width	: 500,
			title	: Sb.Locale.get('seoOwnershipTitle'),
			modal	: true,

			items: {
				xtype		: 'form',
				id			: 'sb-ownership-form',
				fileUpload	: true,
				border		: false,
				defaults	: {layout: 'column', border: false},
				items: [{ //row1
					defaults : {border: false},
					items: [{
						xtype		: 'radio',
						name		: 'ownershipType',
						inputValue	: 'emptyFile',
						hideLabel	: true,
						width		: 20,
						checked		: true
					}, {
						columnWidth: 1,
						layout: 'form',
						labelAlign	: 'top',
						items: [{
							xtype		: 'textfield',
							fieldLabel	: Sb.Locale.get('createFile'),
							name		: 'emptyFilename',
							anchor		: '100%',
							msgTarget	: 'under'
						}, {
							xtype		: 'label',
							text		: Sb.Locale.get('createFileDescription', Sb.Config.general.product),
							cls			: 'sb-editor-box-description'
						}]
					}]
				}, { //row2
					defaults : {border: false},
					style: 'margin-top: 15px',
					items: [{
						xtype		: 'radio',
						name		: 'ownershipType',
						id			: 'sb-ownershipType-customFile',
						inputValue	: 'customFile',
						hideLabel	: true,
						width		: 20
					}, {
						columnWidth: 1,
						layout: 'form',
						labelAlign: 'top',
						items: [{
							xtype			: 'fileuploadfield',
							anchor			: '100%',
							buttonText		: Sb.Locale.get('browseButtonText'),
							fieldLabel		: Sb.Locale.get('customFile'),
							emptyText		: Sb.Locale.get('clickToBrowseAndUploadFile'),
							labelSeparator	: '',
							name			: 'customFile',
							msgTarget		: 'under',
							listeners		: {
								fileselected: function () {
									Ext.getCmp('sb-ownershipType-customFile').setValue(true);
								}
							}
						}, {
							xtype	: 'label',
							text	: Sb.Locale.get('customFileDescription'),
							cls		: 'sb-editor-box-description'
						}]
					}]
				}]
			},
			buttons: [{
				text: Sb.Locale.get('ok'),
				handler: function () {
					var form = Ext.getCmp('sb-ownership-form').getForm();
					if (form.findField('ownershipType').getGroupValue() == 'emptyFile'
						&& !form.findField('emptyFilename').getValue()
					) {
						form.findField('emptyFilename').markInvalid(Sb.Locale.get('fieldCantBeEmpty'));
						return;
					}
					if (form.findField('ownershipType').getGroupValue() == 'customFile'
						&& !form.findField('customFile').getValue()
					) {
						form.findField('customFile').markInvalid(Sb.Locale.get('fieldCantBeEmpty'));
						return;
					}
					form.submit({
						url: Sb.Config.baseUrl + '/Upload/OwnershipVerification/',
						waitMsg: Sb.Locale.get('creatingFile'),
						success: function (fp, action) {
							Sb.Editor.Ui.mainWindow.statusBar.setMessage(
								Sb.Locale.get('verificationFileUploadSuccess')
							);
							form.el.dom.reset();
							ownershipWin.close();
						},
						failure: function (fp, action) {
							var result = action.result;
							Sb.Editor.Ui.mainWindow.statusBar.setMessageError(
								result.message ?
									(result.localized ? result.message : Sb.Locale.get(result.message))
									: Sb.Locale.get('verificationFileUploadFail')
							);
							ownershipWin.close();
						}
					});
				}
			}, {
				text: Sb.Locale.get('cancelButtonTitle'),
				handler: function () {
					ownershipWin.close();
				}
			}]
		});
		ownershipWin.show();
	},

	editHeadWinOpen: function () {
		var tmpField	= this.form.findField('customHead'),
			editHeadWin = new Sb.Window({
			layout		: 'form',
			width		: 500,
			title		: Sb.Locale.get('seoEditHeadTitle'),
			modal		: true,
			labelAlign	: 'top',
			items: {
				xtype	: 'textarea',
				fieldLabel: Sb.Locale.get('addHeaderCode'),
				anchor	: '100%',
				name	: 'customHeadArea',
				value	: tmpField.getValue(),
				msgTarget: 'under'
			},
			buttons: [{
				text: Sb.Locale.get('Add'),
				scope: this,
				handler: function () {
					var field = editHeadWin.find('customHead')[0],
						value = field.getValue();

					if (!this.isValidHead(value)) {
						field.markInvalid(Sb.Locale.get('addHeaderCodeError'));
						return;
					}

					tmpField.setValue(value);
					Sb.editor.UiContext.registerChanges();
					editHeadWin.close();
				}
			}, {
				text: Sb.Locale.get('cancelButtonTitle'),
				handler: function () {
					editHeadWin.close();
				}
			}]
		});
		editHeadWin.show();
	},


	isValidHeadNode: function (node) {
		var allowNodeTypes = [Node.ELEMENT_NODE, Node.TEXT_NODE, Node.CDATA_SECTION_NODE, Node.COMMENT_NODE],
			allowTags = ['base', 'basefont', 'bgsound', 'link', 'meta', 'script', 'style', 'title'];

		if (allowNodeTypes.indexOf(node.nodeType) == -1) {
			return false;
		}

		if (node.nodeType == Node.TEXT_NODE && node.nodeValue.trim().length) {
			return false;
		}

		if (node.nodeType == Node.ELEMENT_NODE && allowTags.indexOf(node.tagName.toLowerCase()) == -1) {
			return false;
		}

		return true;
	},

	isValidHead: function (content) {
		var tmpNode = document.createElement("span"),
			currentNode, i;

		tmpNode.innerHTML = content;

		if (tmpNode.hasChildNodes()) {
			for (i = 0; i < tmpNode.childNodes.length; i++) {
				currentNode = tmpNode.childNodes[i];
				if (!this.isValidHeadNode(currentNode)) {
					return false;
				}
			}
		}

		return true;
	},

	googleAnWinOpen: function () {
		var tmpField	= this.form.findField('googleAnalyticsCode'),
			googleAnWin = new Sb.Window({
			layout		: 'form',
			width		: 500,
			title		: Sb.Locale.get('seoGoogleAnTitle'),
			modal		: true,
			labelAlign	: 'top',
			items: {
				xtype: 'textarea',
				fieldLabel: Sb.Locale.get('addGoogleAnCode', Sb.Config.googleAnalyticsUrl),
				anchor	: '100%',
				height	: 200,
				name	: 'googleAnalyticsCode',
				value	: tmpField.getValue()
			},
			buttons: [{
				text: Sb.Locale.get('Add'),
				scope: this,
				handler: function () {
					tmpField.setValue(googleAnWin.find('googleAnalyticsCode')[0].getValue());
					Sb.editor.UiContext.registerChanges();
					googleAnWin.close();
				}
			}, {
				text: Sb.Locale.get('cancelButtonTitle'),
				handler: function () {
					googleAnWin.close();
				}
			}]
		});
		googleAnWin.show();
	},

	editRobotsTxtWinOpen: function () {
		var tmpField	= this.form.findField('robotsTxt'),
			win = new Sb.Window({
			layout		: 'form',
			width		: 500,
			title		: Sb.Locale.get('seoEditRobotsTxtTitle'),
			modal		: true,
			labelAlign	: 'top',
			items: {
				xtype	: 'textarea',
				ref		: 'codeField',
				fieldLabel: Sb.Locale.get('addRobotsTxtCode'),
				anchor	: '100%',
				height	: 200,
				value	: tmpField.getValue()
			},
			buttons: [{
				text: Sb.Locale.get('Add'),
				handler: function () {
					tmpField.setValue(win.codeField.getValue());
					Sb.editor.UiContext.registerChanges();
					win.close();
				}
			}, {
				text: Sb.Locale.get('cancelButtonTitle'),
				handler: function () {
					win.close();
				}
			}]
		});
		win.show();
	},

	cookieBlockerCodeWinOpen: function () {
		var tmpField = this.form.findField('cookieBlockerCode');
		var win = new Sb.Window({
			layout		: 'form',
			width	: 500,
			title	: Sb.Locale.get('seoCookiePolicyWidgetTitle'),
			modal	: true,
			labelAlign	: 'top',
			items: {
				xtype: 'textarea',
				fieldLabel: Sb.Locale.get('addCookieBlockerCode', Sb.Config.cookiebotUrl),
				anchor	: '100%',
				height	: 200,
				name	: 'cookieBlockerCode',
				value	: tmpField.getValue()
			},
			buttons: [{
				text: Sb.Locale.get('Add'),
				scope: this,
				handler: function () {
					tmpField.setValue(win.find('cookieBlockerCode')[0].getValue());
					Sb.editor.UiContext.registerChanges();
					win.close();
				}
			}, {
				text: Sb.Locale.get('cancelButtonTitle'),
				handler: function () {
					win.close();
				}
			}]

		});
		win.show();
	},

	seoTabAfterRender: function (p) {
		var tab = p.el;
		tab.child('.sb-ownership-link').on('click', this.ownershipWinOpen, this);
		tab.child('.sb-edit-head-link').on('click', this.editHeadWinOpen, this);
		if (Sb.Config.googleAnalyticsEnabled) {
			tab.child('.sb-google-analitics-link').on('click', this.googleAnWinOpen, this);
		}
		tab.child('.sb-edit-robots-txt-link').on('click', this.editRobotsTxtWinOpen, this);
		tab.child('.sb-cookie-policy-widget-link').on('click', this.cookieBlockerCodeWinOpen, this);
	},

	getLocalesFromForm: function () {
		var formValues = this.form.getFieldValues(),
			locales = Ext.isArray(formValues.locale) ? formValues.locale : [formValues.locale];
		return Ext.unique(locales);
	},

	/**
	 * Save handler.
	 * @param {Function} afterSaveCallbacks Handler, that will be run after save
	 */
	saveHandler: function (afterSaveCallbacks) {
		var formValues = this.form.getFieldValues(),
			locales = this.getLocalesFromForm(),
			localeChanged = Sb.Editor.model.get('locale') != locales[0],
			languageWidgetType = Sb.WidgetTypeManager.get('language');

		this.model.changeLocale(Sb.Editor.siteView.getCurrentSiteLocale());

		if (locales.indexOf(Sb.Editor.siteView.getCurrentSiteLocale()) === -1) {
			Ext.util.Cookies.set('currentSiteLocale', locales[0]);
			localeChanged = true;
		}

		if (locales.length > 1 &&
			Ext.util.JSON.encode(Sb.Editor.model.get('locales')) != Ext.util.JSON.encode(locales) &&
			languageWidgetType.cache.length === 0
		) {
			//Add language widget
			Sb.Editor.model.rootPage.widget.widgets.get('header').createChild(languageWidgetType, 0);
		}

		Sb.Editor.model.unserialize({
			pagesUseName				: formValues.pagesUseName,
			customHead					: formValues.customHead,
			googleAnalyticsCode			: formValues.googleAnalyticsCode,
			cookieBlockerCode			: formValues.cookieBlockerCode,
			robotsTxt					: formValues.robotsTxt,
			isFacebookIntegration		: formValues.isFacebookIntegration,
			locale						: locales[0],
			locales						: locales,
			cookiePolicyWidgetEnabled	: formValues.cookiePolicyWidgetEnabled,
			mobileOptimizationsEnabled	: formValues.mobileOptimizationsEnabled,
			faviconFile					: this.faviconFile,
			ownerInfo					: {
				personalName: formValues.siteOwnerInfoPersonalName,
				companyName: formValues.siteOwnerInfoCompanyName,
				phone: formValues.siteOwnerInfoPhone,
				email: formValues.siteOwnerInfoEmail,
				address: formValues.siteOwnerInfoAddress,
				city: formValues.siteOwnerInfoCity,
				state: formValues.siteOwnerInfoState,
				zip: formValues.siteOwnerInfoZip,
				country: formValues.siteOwnerInfoCountry
			}
		});
		var settingsPanel = this;
		Sb.Editor.model.save({
			success: function () {
				Ext.callback(afterSaveCallbacks.success);
				Sb.Editor.Ui.mainWindow.statusBar.setMessage(
					Sb.Locale.get('messageSiteSettingsSaved')
				);
				if (localeChanged) {
					Sb.Loading.updateMessage(Sb.Locale.get('loaderMsgChangingSiteLanguage'));
					Sb.Loading.show();
					window.location.reload();
				}
				if ('update' == Sb.Editor.model.data.faviconFile) {
					Sb.Editor.model.unserialize({
						faviconFile: 'favicon.ico'
					});
				} else if ('remove' == Sb.Editor.model.data.faviconFile) {
					Sb.Editor.model.unserialize({
						faviconFile: null
					});
				}
				settingsPanel.faviconFile = Sb.Editor.model.data.faviconFile;
				if (settingsPanel.uploadForm) {
					settingsPanel.uploadForm.reset();
				}

				var placeholders = Ext.apply({
					siteName: Sb.Editor.model.get('companyName')
				}, Sb.Editor.model.get('ownerInfo'));
				Ext.each(Ext.query('[data-sb-placeholder]', Sb.SiteView.el), function (el) {
					el = Ext.fly(el);
					var name = el.getAttribute('data-sb-placeholder');
					el.update(placeholders[name] || Sb.Locale.get(name + 'DefaultPlaceholder'));
				});

				Sb.Editor.Ui.mainWindow.updatePublishButton();
			},
			failure: afterSaveCallbacks.failure
		});
	},

	changeLocale: function (localeCode) {
		this.model.changeLocale(localeCode);
		this.model.translatableFields.forEach(function (name) {
			this.form.findField(name).setValue(this.model.get(name));
		}, this);
	},

	/**
	 * Discard handler
	 */
	discardHandler: function () {
		this.model.changeLocale(Sb.Editor.siteView.getCurrentSiteLocale());

		this.model.restoreFromMemento(this.memento);
		this.faviconFile = Sb.Editor.model.data.faviconFile;
		if (this.uploadForm) {
			this.uploadForm.reset();
		}
	},

	onFaviconSelected: function (field, fieldValue) {
		if ('' === fieldValue) {
			return;
		}
		if (!this.uploadForm) {
			this.uploadForm = new Ext.form.BasicForm('sb-favicon-form', {
				fileUpload: true
			});
			this.uploadForm.add(field);
		}
		Sb.editor.UiContext.registerChanges();
		var settingsPanel	= this,
			form				= this.uploadForm;
		form.submit({
			url: Sb.Config.baseUrl + '/Upload/Favicon/',
			waitMsg: Sb.Locale.get('uploadingImage'),
			success: function (fp, action) {
				Ext.getDom('sb-favicon-preview').innerHTML = '<img src="' +
					Sb.Utils.antiCache(Sb.Editor.model.getUrl() + '/favicon_new.png', true) + '">';
				settingsPanel.faviconFile = 'update';
				field.resetOnUpload();
			},
			failure: function (fp, action) {
				var message = '';
				if (action.result && action.result.message) {
					message = Sb.Locale.get(action.result.message);
				} else {
					message = Sb.Locale.get('messageNotIcoFile');
				}
				field.markInvalid(message);
			}
		});
	},

	addLanguageField: function (value) {
		var languageField;

		function swapLanguageFieldsValue(field1, field2) {
			Sb.editor.UiContext.registerChanges();
			var tmp = field1.comboField.getValue();
			field1.comboField.setValue(field2.comboField.getValue(), false);
			field2.comboField.setValue(tmp, false);
		}

		languageField = this.languageTab.insert(this.languageTab.items.length - 2, {
			xtype: 'compositefield',
			getValue: function () {
				return this.comboField.getValue();
			},
			name: 'locale',
			isLngField: true,
			hideLabel: true,
			items: [this.getLanguageComboCfg(value), {
				xtype: 'button',
				iconCls: 'sb-up-icon',
				handler: function () {
					swapLanguageFieldsValue(languageField, languageField.previousSibling());
				}
			}, {
				xtype: 'button',
				iconCls: 'sb-down-icon',
				ref: 'downbutton',
				disabled: true,
				handler: function () {
					swapLanguageFieldsValue(languageField, languageField.nextSibling());
				}
			}, {
				xtype: 'button',
				iconCls: 'sb-delete-icon',
				scope: this,
				handler: function () {
					Ext.Msg.show({
						title: Sb.Locale.get('siteLanguageDeleteTitle'),
						msg: Sb.Locale.get('siteLanguageDeleteConfirm'),
						closable: false,
						buttons: Ext.Msg.YESNO,
						scope: this,
						fn: function (btn) {
							if (btn == 'yes') {
								Sb.editor.UiContext.registerChanges();
								var previousCmp = languageField.previousSibling(),
									nextCmp = languageField.nextSibling(),
									hasNextLngField = nextCmp && nextCmp.isLngField;

								if (languageField.downbutton.disabled && !previousCmp.isFirstLngField) {
									previousCmp.downbutton.disable();
								}

								this.form.remove(languageField.comboField);
								this.languageTab.remove(languageField, true);

								if (previousCmp.isFirstLngField && !hasNextLngField) {
									this.languageTab.mainLanguageField.labelBox.hide();
									this.languageTab.mainLanguageField.doLayout();
								}
								this.updateAddLocaleButton();
							}
						}
					});
				}
			}]
		});

		this.languageTab.mainLanguageField.labelBox.show();

		if (languageField.previousSibling().downbutton) {
			languageField.previousSibling().downbutton.enable();
		}

		this.languageTab.doLayout();
	},

	getLanguageComboCfg: function (value) {
		value = value || Sb.Editor.model.get('locale');

		return {
			flex: 5,
			xtype: 'iconcombo',
			editable: false,
			mode: 'local',
			shadow: false,
			store: 'locales',
			valueField: 'code',
			displayField: 'name',
			iconStyleField: 'iconStyle',
			value: value,
			lazyInit: false,
			ref: 'comboField',
			listeners: {
				scope: this,
				select: function () {
					Sb.editor.UiContext.registerChanges();
					this.updateAddLocaleButton();
				},
				afterrender: function (c) {
					Ext.each([true, false], function (hasTranslation) {
						var index = c.store.find('hasTranslation', hasTranslation);
						if (index == -1) {
							return true;
						}
						Ext.DomHelper.insertBefore(c.view.getNode(index), {
							cls: 'x-combo-list-group',
							html: Sb.Locale.get('siteLanguage' +
								(hasTranslation ? 'WithTranslation' : 'WithoutTranslation'))
						});
					});
				}
			}
		};
	},

	updateAddLocaleButton: function () {
		var localesFromForm = this.getLocalesFromForm(),
			checkLocales = Sb.Limits.checkLocales(this.model, localesFromForm);

		if (checkLocales.isExceeded) {
			Sb.Editor.Ui.mainWindow.statusBar.setMessageWarning(
				Sb.Locale.get(
					'maxAdditionalLocalesLimitExceeded',
					localesFromForm.length - 1,
					checkLocales.limit
				)
			);
		}

		Ext.getCmp('addLocaleButton')[checkLocales.isReached ? 'addClass' : 'removeClass']('sb-upsell-button');
	}
});