/**
 * Base class for all component edit panels
 *
 * @class Sb.editor.component.BaseStylePanel
 * @extends Ext.Panel
 */
Ext.define('Sb.editor.component.BaseStylePanel', {
	extend: 'Ext.Panel',

	autoRender: true,
	baseCls: 'sb-widget-edit-panel',
	frame: false,

	/**
	 * @cfg {null|Array} offsets
	 * Array of coordinates for edit panel position offsets relative to widget.
	 */
	offsets: null,

	afterRender: function () {
		var stylePanel = this;

		Sb.editor.component.BaseStylePanel.superclass.afterRender.call(this);

		if (this.component.movable) {
			this.el.addClass('movable');

			this.moveControl = this.el.createChild({
				cls: 'sb-widget-edit-panel-move',
				html: '&nbsp;'
			}, this.el.first());

			this.dd = new Ext.dd.DD(this, null, {
				moveOnly: true
			});
			this.dd.editPanel = this;
			this.dd.setHandleElId(this.moveControl);
			this.dd.startDrag = function () {
				stylePanel.toFront();
				this.constrainTo(Ext.getBody());
				this.iframeFix = Ext.DomHelper.append(document.body, {tag: 'div', cls: 'draggable-iframe-overlay'});
			};
			this.dd.endDrag = function () {
				this.editPanel.offsets = this.editPanel.el.getOffsetsTo(this.editPanel.component.el);
				document.body.removeChild(this.iframeFix);
			};
		}

		this.el.on('click', this.toFront, this);
	},

	initComponent: function () {
		Ext.apply(this, {
			cls: 'sb-widget-style-panel'
		});
		Sb.editor.component.BaseStylePanel.superclass.initComponent.apply(this, arguments);
	},

	show: function (component) {
		this.component = component;
		Sb.editor.component.BaseStylePanel.superclass.show.call(this);
		this.reanchor();

		this.component.toFront();
		this.toFront();
	},

	hide: function () {
		Sb.editor.component.BaseStylePanel.superclass.hide.call(this);
		this.offsets = null;
	},

	reanchor: function (realign) {
		if (Sb.Editor.Ui.isDragActive() || !this.rendered) {
			return;
		}

		realign = Ext.isDefined(realign) ? realign : true;

		var offsetX = 1,
			rightSpace,
			leftSpace = this.component.el.getLeft() - offsetX,
			width = this.el.getWidth(),
			height = this.el.getHeight(),
			x = leftSpace - width,
			y = this.component.el.getY(),
			componentRight = this.component.el.getRight(),
			screenWidth = Ext.get('page').getWidth(),
			viewHeight = Ext.lib.Dom.getViewHeight(),
			maxX = screenWidth - width,
			maxY = Ext.lib.Dom.getViewHeight() - height,
			isComponentVisible = y < viewHeight && this.component.el.getBottom() > 0;

		this.el.setVisible(isComponentVisible);
		if (!isComponentVisible) {
			return;
		}

		if (this.offsets) {
			x = this.component.el.getX() + this.offsets[0];
			y += this.offsets[1];
			this.el.setXY([x, y]);

			if (this.el.getTop() < 0) {
				this.el.setY(0);
			}

			if (y > viewHeight) {
				this.el.setY(viewHeight - this.el.getHeight());
			}

			return;
		}

		if (realign) {
			rightSpace = screenWidth - componentRight - offsetX;
			if (leftSpace < width && rightSpace > leftSpace) {
				this.position = 'r';
				x = componentRight + offsetX;
			} else {
				this.position = 'l';
			}
		} else if (this.position == 'r') {
			x = componentRight + offsetX;
		}

		if (x < 0) {
			x = 0;
		}

		if (x > maxX) {
			x = maxX;
		}

		if (y < 0) {
			y = 0;
		}

		if (y > maxY) {
			y = maxY;
		}

		this.el.setXY([x, y]);
	},

	toFront: function () {
		var editPanel = this.component.getEditPanel(),
			zIndex = parseInt(this.component.el.getStyle('z-index'), 10);

		this.el.setStyle('z-index', zIndex + 2);
		if (editPanel && editPanel.rendered) {
			editPanel.el.setStyle('z-index', zIndex + 1);
		}
	}
});