/**
 * @class Sb.editor.component.Date
 * @extends Sb.editor.component.Base
 * Class for image component
 * @constructor
 * @param {Object} config The config object
 * @xtype date_component
 */
Ext.define('Sb.editor.component.Date', {
	extend: 'Sb.editor.component.Base',
	xtype: 'date_component',

	cls: 'sb-component-date',

	/**
	 * @cfg {Function} renderer
	 * <p>Optional. A renderer is an 'interceptor' method which can be used transform value (date) before it is rendered.</p>
	 * <p>The function should return a string value.</p>
	 */
	renderer: Ext.util.Format.date,

	onRender: function () {
		Sb.editor.component.Date.superclass.onRender.apply(this, arguments);
		this.setValue(this.value);
	},

	getEditPanel: function () {
		if (!Sb.editor.component.Date.editPanel) {
			Sb.editor.component.Date.editPanel = new Sb.editor.component.DateEditPanel();
		}
		return Sb.editor.component.Date.editPanel;
	},

	onStartEdit: function () {
		Sb.editor.component.Date.superclass.onStartEdit.apply(this);
		this.originalValue = this.value;
	},

	onCancelEdit: function () {
		Sb.editor.component.Image.superclass.onCancelEdit.apply(this);
		this.setValue(this.originalValue);
	},

	/**
	 * Sets the value of the component
	 * @param {String/Date} value The date or valid date string
	 * @return {Sb.editor.component.Date} this
	 */
	setValue: function (value) {
		this.value = value;
		if (this.rendered) {
			this.update(this.renderer(this.value));
		}
		return this;
	},

	/**
	 * Gets the value of the component
	 * @return {Date} 
	 */
	getValue: function (value) {
		return this.value;
	}
});