/**
 * @class Sb.form.ColorDecorationPanel
 * @extends Sb.form.FieldGroup
 */
Ext.define('Sb.form.ColorDecorationPanel', {
	extend: 'Sb.form.FieldGroup',

	/**
	 * Possible decoration values/keys
	 */
	decorations: ['Bold', 'Italic', 'Underline'],

	constructor: function (config) {
		config = Ext.apply({
			colorComboWidth	: 'auto',
			widgetControlCls	: '',
			children		: 'color,background,decoration'
		}, config);

		Sb.form.ColorDecorationPanel.superclass.constructor.call(this, config);

		var reChildren	= new RegExp(config.children.replace(/,/g, '|'), 'i');

		if (reChildren.test('color')) {
			var colorCombo = new Sb.form.ColorDropDown({
				itemId			: 'Color',
				fieldLabel		: Sb.Locale.get('colorDecorationPanelColor'),
				itemCls			: config.widgetControlCls + 'color',
				width			: config.colorComboWidth,
				allowColorPalette: true
			});
			this.add(colorCombo);
		}

		if (reChildren.test('background')) {
			var backgroundCombo = new Sb.form.ColorDropDown({
				itemId			: 'BackgroundColor',
				iconCls			: 'x-edit-backcolor',
				fieldLabel		: Sb.Locale.get('colorDecorationPanelBackgroundArea'),
				itemCls			: config.widgetControlCls + 'background-color',
				isShowColorField: false,
				width			: config.colorComboWidth,
				noColorText		: this.noColorText,
				allowBlank		: true
			});
			this.add(backgroundCombo);
		}

		if (reChildren.test('decoration')) {
			var decorationGroup = new Sb.form.FieldGroup({
				itemId		: 'Decoration',
				layout		: 'column',
				fieldLabel	: Sb.Locale.get('colorDecorationPanelDecoration'),
				itemCls		: config.widgetControlCls + 'decoration'
			});
			Ext.each(this.decorations, function (decoration) {
				var decorationButton = new Ext.Button({
					itemId		: decoration,
					iconCls		: 'x-edit-' + decoration.toLowerCase(),
					enableToggle: true
				});
				decorationGroup.add(decorationButton);
			});
			this.add(decorationGroup);
		}
	}
});