/**
 * @class Sb.forms.LanguageButtons
 * @extends Ext.DataView
 * @xtype languagebuttons
 */
Ext.define('Sb.form.LanguageButtons', {
	extend: 'Ext.DataView',
	xtype: 'languagebuttons',

	initComponent: function () {
		Ext.apply(this, {
			autoEl: 'span',
			cls: 'sb-language-buttons',
			tpl: new Ext.XTemplate(
				'<tpl for=".">',
				'<span class="sb-widget-control-locale" ' +
					'style="background-image: url(' + Sb.Config.siteResourcesUrl +
					'/images/flags/16/{countryCode}.png);" title="{title}"></span>',
				'</tpl>'
			),
			itemSelector: 'span',
			selectedClass: 'selected',
			singleSelect: true,
			store: new Ext.data.ArrayStore({
				fields: ['code', 'countryCode', 'title']
			})
		});

		if (this.data) {
			this.setData(this.data);
			delete this.data;
		}

		this.callParent();
	},

	setData: function (data) {
		this.suspendEvents();
		this.store.loadData(data.map(function (code) {
			return [
				code,
				code.split('_')[1].toLowerCase(),
				Sb.Config.locales[code].name
			];
		}), false);
		this.resumeEvents();
		return this;
	},

	afterRender: function () {
		this.callParent();

		if (this.value) {
			this.setValue(this.value);
		}
	},

	onContainerClick: Ext.emptyFn,

	getValue: function () {
		var records;

		if (this.rendered) {
			records = this.getSelectedRecords();
			if (records.length > 0) {
				return records[0].get('code');
			}
		}

		return this.value;
	},

	setValue: function (value) {
		this.value = value;
		if (this.rendered) {
			this.select(this.store.find('code', value), false, true);
		}
		return this;
	}
});
