/**
 * @class Sb.model.VideoWidget
 * @extends Sb.model.ScriptWidget
 * Class for video model
 * @constructor
 * @param {Object} config The configuration options for the widget model
 */
Ext.define('Sb.model.VideoWidget', {
	extend: 'Sb.model.ScriptWidget',

	/**
	 * @cfg {String} type
	 * defaults to <tt>video</tt>
	 */

	constructor: function (config) {
		config = Ext.apply({
			type: 'video'
		}, config);

		this.addEvents(
			/**
			 * @event urlchange
			 * Fires after url has changed.
			 */
			'urlchange'
		);

		/**
		 * Unknown provider
		 *
		 * @var Object
		 */
		this.unknownProvider = {label: 'Unknown', isKnown: false};

		/**
		 * Empty provider
		 *
		 * @var Object
		 */
		this.emptyProvider = {label: 'Empty', isKnown: false};

		/**
		 * Current calculated provider
		 *
		 * @var Object
		 */
		this.currentProvider = this.emptyProvider;

		Sb.model.VideoWidget.superclass.constructor.call(this, config);

		this.on('urlchange', this.onUrlChange, this);
	},

	onUrlChange: function () {
		var url		= this.data.url,
			isEmpty	= '' === url,
			code	= '',
			isValid	= Ext.form.VTypes.url(url);

		this.currentProvider = isEmpty ? this.emptyProvider : this.unknownProvider;
		if (!isEmpty && isValid) {
			Ext.each(this.getType().getProviders(), function (provider) {
				if (url.indexOf(provider.host + '/') == -1) {
					return true;
				}
				var videoMatch = url.match(provider.videoRe);
				if (null == videoMatch) {
					return false;
				}
				this.currentProvider = provider;
				code = provider.template.apply({
					width	: provider.width,
					height	: provider.height,
					videoId	: videoMatch[1],
					lang	: Sb.Config.locale
				});
				return false;
			}, this);
		}

		this.unserialize({
			autoCode: code,
			provider: this.currentProvider.label
		});
	},

	/**
	 * Gets current widget`s code: auto or default
	 *
	 * @returns {String}
	 */
	getCode: function () {
		return this.data[1 == this.data.isCustom ? 'code' : 'autoCode'];
	},

	/**
	 * Fires on changing of single property change to allow us fire up navigation widget change events.
	 * @param {String} key
	 */
	onUnserializePropertyChange: function (key) {
		this.updateIsConfiguredProperties();
		if ('url' == key) {
			this.fireEvent('urlchange');
		}
	}
});