/**
 * @class Sb.widget.Breadcrumbs
 * @extends Sb.widget.Widget
 * Class for Breadcrumbs widget
 * @xtype breadcrumbs_widget
 */
Ext.define('Sb.widget.Breadcrumbs', {
	extend: 'Sb.widget.Widget',
	xtype: 'breadcrumbs_widget',
	translatable: true,

	/**
	 * @cfg {Sb.widget.Breadcrumbs} model
	 * The {@link Sb.widget.Breadcrumbs} to bind this Widget to.
	 */

	cls			: 'sb-widget-breadcrumbs',

	onRender: function () {
		Sb.widget.Breadcrumbs.superclass.onRender.apply(this, arguments);
		this.refresh();
	},

	refresh: function () {
		this.refreshCode();
		this.refreshStyle();
	},

	refreshCode: function () {
		var path = Sb.Editor.siteView.currentPage.getPath(),
			data = this.model.data,
			encode = Ext.util.Format.htmlEncode,
			iconCls = data.startPageTitle ? 'icon-with-title' : 'icon';

		path[0] = Sb.Editor.model.rootPage.getFirstVisiblePage();

		this.cleanInternalListeners();
		this.update('');
		if (data.isShowStartIcon) {
			this.widgetContentEl.createChild({
				tag: 'a',
				cls: iconCls,
				href: '#'
			});
		}
		Ext.each(path, function (page, index) {
			var isFirst			= 0 === index,
				title			= encode(isFirst ? data.startPageTitle : page.data.title),
				itemEl			= null,
				isShowTitle		= '' !== title,
				isShowSeparator	= true;

			if (isFirst && !data.isShowStartIcon && !isShowTitle) {
				isShowSeparator = false;
			}

			if (path[index + 1]) {
				if (isShowTitle) {
					itemEl = this.widgetContentEl.createChild({
						tag: 'a',
						cls: 'page',
						href: '#',
						html: title
					});
					this.addInternalListener(itemEl, 'click', function (event, el, options) {
						event.stopEvent();
						Sb.Editor.siteView.setCurrentPage(options.page);
					}, null, {'page' : page});
				}
				if (isShowSeparator) {
					this.widgetContentEl.createChild({
						tag: 'span',
						cls: 'separator',
						html: Ext.util.Format.htmlEncode(data.separator)
					});
				}
			} else {
				itemEl = this.widgetContentEl.createChild({
					tag: 'span',
					cls: 'page',
					html: title
				});
			}

			if (null != itemEl) {
				this.addInternalListener(page, 'change', function () {
					title = encode(isFirst ? data.startPageTitle : page.data.title);
					itemEl.update(title);
				});
			}
		}, this);
	},

	/**
	 * Updates CSS for breadcrumbs widget.
	 */
	refreshStyle: function () {
		var model				= this.model,
			fontSize			= model.data.fontSize,
			iconHeight			= 16,
			maxLineHeightRatio	= 1.44, //ratio for Arial Black as biggest font
			lineHeight			= Math.ceil(Math.max(fontSize * maxLineHeightRatio, iconHeight)),
			data				= Ext.apply({
				className	: '#widget-' + this.model.data.uuid + ' .widget-content',
				lineHeight	: lineHeight
			}, model.data);

		data.fontColor = Sb.Editor.model.design.getColor(data.fontColor);

		Sb.StyleManager.addStyleSheet(
			this.getType().getCssTemplate().apply(data),
			'widget-breadcrumbs',
			this.model.data.uuid
		);

		Sb.FontManager.loadFont(data.fontFamily);

		if (Sb.Editor.Ui.editComponent === this) {
			this.getEditPanel().reanchor();
		}
	},

	initEvents: function () {
		Sb.widget.Breadcrumbs.superclass.initEvents.call(this);

		this.mon(Sb.Editor.model.rootPage, 'change', this.refreshCode, this);
		this.mon(this.model, 'propertychange', function (key) {
			if (/^font(Family|Size|Color)|decoration(Bold|Italic|Underline)$/.test(key)) {
				this.refreshStyle();
			} else {
				this.refreshCode();
			}
		}, this);
		this.mon(Sb.Editor.model.design, 'colorschemechange', this.refreshStyle, this);
	},

	onDocumentClick: function (event) {
		if (event.getTarget('.x-combo-list') || event.getTarget('.sb-color-dropdown')) {
			return;
		}

		Sb.widget.Breadcrumbs.superclass.onDocumentClick.call(this, event);
	}
});