/**
 * Allows to edit breadcrumbs.
 *
 * @class Sb.widget.BreadcrumbsEditPanel
 * @extends Sb.widget.WidgetEditPanel
 */
Ext.define('Sb.widget.BreadcrumbsEditPanel', {
	extend: 'Sb.widget.WidgetEditPanel',

	initComponent: function () {
		Ext.apply(this, {
			layout		: 'form',
			labelAlign	: 'top',
			autoHeight	: true,
			itemCls		: 'sb-widget-edit-panel-breadcrumbs-form',
			cls			: 'sb-widget-edit-panel-breadcrumbs',
			items		: []
		});

		Sb.widget.BreadcrumbsEditPanel.superclass.initComponent.apply(this, arguments);
	},

	onRender: function () {
		var topGroup		= new Sb.form.FieldGroup(),
			bottomGroup		= new Sb.form.FieldGroup();

		topGroup.addClass('clearfix');
		bottomGroup.addClass('clearfix');

		/**
		 * Start page title field
		 *
		 * @type Ext.form.TextField
		 * @property startPageTitle
		 */
		this.startPageTitle = new Ext.form.TextField({
			fieldLabel		: Sb.Locale.get('breadcrumbsWidgetStartPageTitle'),
			itemCls			: 'sb-breadcrumbs-start-page-control',
			width			: 180,
			maxLength		: 255,
			enableKeyEvents	: true,
			listeners		: {
				scope: this,
				keyup: function (field, event) {
					this.reanchor();
				}
			}
		});

		/**
		 * Separator selection combo, that allows to enter custom values
		 *
		 * @type Ext.form.ComboBox
		 * @property separator
		 */
		this.separator = new Ext.form.ComboBox({
			fieldLabel	: Sb.Locale.get('breadcrumbsWidgetSeparator'),
			store		: ['>', '>>', '»', '›', '→', '/', '::', '•'],
			enableKeyEvents: true,
			mode		: 'local',
			itemCls		: 'sb-breadcrumbs-separator-control',
			width		: 60
		});

		/**
		 * Do we need to show start icon checkbox
		 *
		 * @type Ext.form.Checkbox
		 * @property isShowStartIcon
		 */
		this.isShowStartIcon = new Ext.form.Checkbox({
			itemCls		: 'sb-breadcrumbs-add-home-control',
			hideLabel	: true,
			boxLabel	: Sb.Locale.get('breadcrumbsWidgetAddHomeIcon'),
			width		: 140
		});

		var fontFamilySizePanel = new Sb.form.FontFamilySizePanel({
			fontFamilyWidth: 120,
			fontSizeWidth: 60
		});

		/**
		 * Font family combobox
		 *
		 * @type Ext.form.Checkbox
		 * @property fontFamily
		 */
		this.fontFamily = fontFamilySizePanel.getComponent('FontFamily');

		/**
		 * Font size combobox
		 *
		 * @type Ext.form.Checkbox
		 * @property fontSize
		 */
		this.fontSize = fontFamilySizePanel.getComponent('FontSize');

		var colorDecorationPanel = new Sb.form.ColorDecorationPanel({
				cls				: 'sb-edit-control clearfix',
				children		: 'color,decoration'
			}),
			fontDecoration = colorDecorationPanel.getComponent('Decoration');

		/**
		 * Font color field.
		 *
		 * @type Sb.form.ColorDropDown
		 * @property fontColor
		 */
		this.fontColor = colorDecorationPanel.getComponent('Color');

		/**
		 * Decoration bold button
		 *
		 * @type Ext.Button
		 * @property decorationBold
		 */
		this.decorationBold = fontDecoration.getComponent('Bold');

		/**
		 * Decoration italic button
		 *
		 * @type Ext.Button
		 * @property decorationItalic
		 */
		this.decorationItalic = fontDecoration.getComponent('Italic');

		/**
		 * Decoration underline button
		 *
		 * @type Ext.Button
		 * @property decorationUnderline
		 */
		this.decorationUnderline = fontDecoration.getComponent('Underline');

		topGroup.add([this.startPageTitle, this.separator, this.isShowStartIcon]);
		bottomGroup.add([fontFamilySizePanel, colorDecorationPanel]);
		this.add([
			topGroup,
			bottomGroup
		]);

		this.on('show', function () {
			this.bindModel([
				'startPageTitle', 'separator', 'fontFamily', 'fontSize', 'isShowStartIcon',
				'fontColor', 'decorationBold', 'decorationItalic', 'decorationUnderline'
			]);
		}, this);

		Sb.widget.BreadcrumbsEditPanel.superclass.onRender.apply(this, arguments);
	},

	onPropertyChange: function (name, value) {
		if (name === 'startPageTitle') {
			this.startPageTitle.setValue(value);
		}
	},

	show: function (widget) {
		this.callParent([widget]);
		this.widget.model.on('propertychange', this.onPropertyChange, this);
	},

	hide: function() {
		this.callParent();
		this.widget.model.un('propertychange', this.onPropertyChange, this);
	}
});