/**
 * @class Sb.widget.Contact
 * @extends Sb.widget.Widget
 * Class for Contact widget
 * @constructor
 * @param {Object} config The config object
 * @xtype contact_widget
 */
Ext.define('Sb.widget.Contact', {
	extend: 'Sb.widget.Widget',
	xtype: 'contact_widget',
	translatable: true,

	cls: 'sb-widget-contact',

	/**
	 * The template to use when rendering the content of Widget.
	 * @property contentTpl
	 * @type Ext.XTemplate
	 */
	contentTpl: null,

	initComponent: function () {
		Sb.widget.Contact.superclass.initComponent.apply(this, arguments);

		this.contentTpl = new Ext.XTemplate(
			'<form id="widget-{uuid}-form">' +
				'<tpl for="fields">',
				'<div class="form-item">' +
					'<label class="form-item-label" for="widget-{parent.uuid}-field-{name}">' +
						'{title:htmlEncode}' +
						'<tpl if="required">' +
							' <span class="form-required">*</span>' +
						'</tpl>' +
					'</label>' +
					'<tpl if="type==\'textfield\' || type==\'email\'">' +
						'<input id="widget-{parent.uuid}-field-{name}" class="form-item-textfield" ' +
							'type="text" name="{name}"' +
						' />' +
					'</tpl>' +
					'<tpl if="type==\'textarea\'">' +
					'<textarea id="widget-{parent.uuid}-field-{name}" class="form-item-textarea" name="{name}"></textarea>' +
					'</tpl>' +
				'</div>' +
				'</tpl>' +
				'<tpl if="captchaEnabled">' +
					'<div class="form-item captcha"<tpl if="captchaHidden"> style="display: none;"</tpl>>' +
						'<label class="form-item-label" for="widget-{uuid}-captcha">' +
						Ext.util.Format.htmlEncode(Sb.Locale.get('widgetContactCaptchaFieldTitle')) +
							' <span class="form-required">*</span>' +
						'</label>' +
						'<div class="form-item-captcha" id="recaptcha-{uuid}"' +
							'<tpl if="captchaContainerHeight"> style="height: {captchaContainerHeight}px;"</tpl>>' +
							'<div class="g-recaptcha" data-sitekey="' + Sb.Config.recaptcha.publicKey + '"></div>' +
						'</div>' +
					'</div>' +
				'</tpl>' +
				'<div class="form-note">' +
					Ext.util.Format.htmlEncode(Sb.Locale.get('widgetContactFormNote', ' * ')) +
				'</div>' +
				'<input type="submit" class="form-submit" value="{buttonText:htmlEncode}" />' +
			'</form>'
		);
	},

	initEvents: function () {
		Sb.widget.Contact.superclass.initEvents.call(this);
		this.mon(this.model, 'change', this.refreshContent, this);
		this.mon(this.model, 'changefields', this.refreshContent, this);
	},

	renderContent: function () {
		var data = this.model.serialize();
		data.captchaHidden = Sb.Editor.Ui.editComponent != this;
		data.captchaContainerHeight = this.captchaContainerHeight;

		this.update(this.contentTpl.apply(data));
	},

	afterRenderContent: function () {
		if (Sb.Editor.Ui.editComponent == this) {
			this.getEditPanel().reanchor();
			this.showCaptcha();
		}
	},

	showCaptcha: function () {
		var captchaContainer;

		if (!this.model.get('captchaEnabled')) {
			return;
		}

		captchaContainer = Ext.get('recaptcha-' + this.model.get('uuid'));
		if (!captchaContainer) {
			return;
		}

		this.el.child('.captcha').setVisibilityMode(Ext.Element.DISPLAY).setVisible(true);

		Sb.ResourceManager.require(['https://www.google.com/recaptcha/api.js'], function () {
			this.getEditPanel().reanchor();
		}, this);
	},

	completeEdit: function (explicitCompletion) {
		var editPanel = this.getEditPanel();

		if (explicitCompletion && !editPanel.recipientsField.isValid()) {
			editPanel.tabs.setActiveTab(0);
			editPanel.recipientsField.validate();
			return;
		}

		Sb.widget.Contact.superclass.completeEdit.call(this, arguments);
	},

	onStartEdit: function () {
		Sb.widget.Contact.superclass.onStartEdit.call(this);
		this.showCaptcha();
	},

	onHideEdit: function () {
		Sb.widget.Contact.superclass.onHideEdit.call(this);
		if (this.model.isConfiguredProperties() && this.model.get('captchaEnabled')) {
			this.el.child('.captcha')
				.setVisibilityMode(Ext.Element.DISPLAY)
				.setVisible(false);
		}
	},

	onCompleteEdit: function () {
		var editPanel = this.getEditPanel();
		if (editPanel && editPanel.recipientsField.isValid()) {
			var recipients = editPanel.recipientsField.getValue()
				.replace(/\s*/g, '')
				.replace(/(,|;)/g, ', ');
			this.model.set('recipients', recipients);
		}

		Sb.widget.Contact.superclass.onCompleteEdit.apply(this, arguments);
	},

	onDocumentClick: function (event) {
		if (event.getTarget('.x-combo-list')) {
			return;
		}

		Sb.widget.Header.superclass.onDocumentClick.call(this, event);
	}
});