/**
 * @class Sb.widget.Imagegallery
 * @extends Sb.widget.Widget
 * Class for Image Gallery widget
 * @constructor
 * @param {Object} config The config object
 * @xtype imagegallery_widget
 */
Ext.define('Sb.widget.Imagegallery', {
	extend: 'Sb.widget.Widget',
	xtype: 'imagegallery_widget',

	cls: 'sb-widget-imagegallery',

	isShowSaveMsg: true,

	currentPage: 1,

	// with a minus index numbering from the end
	lastPage: -1,

	// jQuery object
	jImageGallery: null,

	imagesDropTarget: null,

	initComponent: function () {
		Sb.widget.Imagegallery.superclass.initComponent.apply(this, arguments);
		this.setupRemovalMessages();
		this.imagesDD = [];
	},

	/**
	 * Updates removal messages depending of current storage type
	 */
	setupRemovalMessages: function () {
		this.removalConfirmationMessage = Sb.Locale.get(
			this.model.isLocal() ? 'widgetImagegalleryRemoveConfirmationLocal' : 'widgetImagegalleryRemoveConfirmation'
		);
	},

	onRender: function () {
		Sb.widget.Imagegallery.superclass.onRender.apply(this, arguments);

		if (this.getType().isAvailable()) {
			var albumId = this.model.get('albumId');

			this.updateToLoading();
			if (albumId) {
				this.model.refreshImages(albumId, this.onImagesRefresh, this.onError, this);
			} else {
				this.onImagesRefresh();
			}

			this.widgetTitle.contentElement.setStyle('text-align', this.model.get('align'));
		} else {
			this.renderPlaceholder();
		}
	},

	renderPlaceholder: function () {
		Sb.widget.Imagegallery.superclass.renderPlaceholder.apply(this, arguments);
		var type = this.getType();
		this.togglePlaceholder(true);
		this.update(this.placeholderTpl.apply({
			placeholderMsg: type.getTranslation('descriptionDisabled' + type.disableReason.toUpperCaseFirst())
		}));
		this.addClass(this.getStateCls('inactive'));
	},

	/**
	 * Updates current widget content to "Loading..." stage when we are working in with remote storage
	 */
	updateToLoading: function () {
		if (this.model.isLocal()) {
			return;
		}
		if (Sb.Editor.Ui.editComponent == this) {
			this.widgetContentEl.setHeight(this.widgetContentEl.getHeight());
		}
		this.togglePlaceholder(false);
		this.update(Sb.Locale.get('widgetImagegalleryLoading'));
		this.reanchorEditPanel();
		this.getType().getEditPanel().toggleStorageTypeControls(false);
	},

	/**
	 * Calls after each images refresh/load:
	 *  - links up resources
	 *  - calls refresh
	 */
	onImagesRefresh: function () {
		this.togglePlaceholder(false);
		var resourcesUrl = Sb.Config.siteResourcesUrl;
		Sb.ResourceManager.require([
			resourcesUrl + '/components/' + Sb.Config.jsFrameworkFile
		], function () {
			Sb.ResourceManager.require([
				Sb.Utils.antiCache(resourcesUrl + '/components/jquery/jquery.jcarousellite.js'),
				Sb.Utils.antiCache(resourcesUrl + '/components/jquery/jquery.simplemodal.js')
			], function () {
				var resources = [
					Sb.Utils.antiCache(resourcesUrl + '/modules/imagegallery/imagegallery.js'),
					Sb.Utils.antiCache(resourcesUrl + '/modules/imagegallery/imagegallery.css')
				];
				if (Sb.Editor.model.isRtl()) {
					resources.push(Sb.Utils.antiCache(resourcesUrl + '/modules/imagegallery/imagegallery-rtl.css'));
				}
				Sb.ResourceManager.require(resources, function () {
					var localeCode = Sb.Config.fallbackLocale;
					if (Sb.Config.locales[Sb.Editor.siteView.getCurrentSiteLocale()].hasTranslation) {
						localeCode = Sb.Editor.siteView.getCurrentSiteLocale();
					}
					Sb.ResourceManager.require([
						Sb.Utils.antiCache(resourcesUrl + '/modules/imagegallery/imagegallery.locale-' +
							localeCode + '.js')
					], this.refresh, this);
				}, this);
			}, this);
		}, this);
	},

	refresh: function () {
		this.widgetContentEl.setHeight();
		this.createGallery();
		this.reanchorEditPanel();
		this.getType().getEditPanel().toggleStorageTypeControls(true);
	},

	initEvents: function () {
		Sb.widget.Imagegallery.superclass.initEvents.call(this);
		this.mon(this.model, 'propertychange', function (key, value, oldValue) {
			if (['thumbSize', 'align', 'albumId', 'storageType', 'pageSize'].indexOf(key) > -1) {
				this.currentPage = 1;
			}

			if ('thumbSize' == key) {
				this.onThumbSizeChange(value, oldValue);
			} else if ('albumId' == key || 'storageType' == key) {
				this.updateToLoading();
				this.model.loadImages(this.model.get('albumId'), this.onImagesRefresh, this.onError, this);
			}
			if ('storageType' == key) {
				this.setupRemovalMessages();
			}
			if ('pageSize' == key) {
				this.refresh();
				this.reanchorEditPanel();
			}
			if ('align' == key) {
				this.onAlignChange(value);
			}
		}, this);
		this.mon(this.model, 'save', function () {
			if (this.isShowSaveMsg) {
				Sb.Editor.Ui.mainWindow.statusBar.setMessage(
					Sb.Locale.get('messageChangesSaved')
				);
			}
		}, this);
	},

	/**
	 * Build image gallery frames
	 */
	createGallery: function () {
		var images = this.model.getImages();

		this.jImageGallery = jQuery(this.widgetContentEl.dom).imagegallery({
			thumbSize: this.model.get('thumbSize'),
			isShowRemove: true,
			pageSize: this.model.get('pageSize'),
			align: this.model.get('align'),
			isChangeHashParams: false,
			isRtl: Sb.Editor.model.isRtl(),
			changeImageCallback: this.onChangeImage.createDelegate(this),
			beforeOpenImageCallback: this.onBeforeOpenImage.createDelegate(this),
			openImageCallback: this.onOpenImage.createDelegate(this),
			closeImageCallback: this.onCloseImage.createDelegate(this),
			changePageCallback: this.changePageHandler.createDelegate(this),
			keyboardShortcuts: false,
			store: {
				type: 'array',
				data: images,
				currentPage: this.currentPage
			}
		});

		if (Ext.isIE) {
			this.el.repaint();
		}
	},

	/**
	 * Change page callback for ImageGallery plugin
	 */
	changePageHandler: function () {
		this.createEmptyFrame();
		this.addRemoveButtons();
		if (this.model.isLocal()) {
			this.addReordering();
		}
		this.anchorControls();
		this.reanchorEditPanel();
		this.reanchorStylePanel();
	},

	/**
	 * Add remove button for images
	 */
	addRemoveButtons: function () {
		var images = this.widgetContentEl.query('[id^="frame-"]');
		Ext.each(images, function (image, index) {
			var frame = Ext.get(image),
				removeButton = frame.first().first().createChild({tag: 'a', cls: 'sb-imagegallery-remove-button'});

			removeButton.on('click', function (e) {
				e.stopEvent();
				this.onImageRemove(image.id);
			}, this);
		}, this);
	},

	/**
	 * Create empty frame with upload support
	 */
	createEmptyFrame: function () {
		var emptyFrame = this.widgetContentEl.first().createChild({
			cls: 'frame frame-empty',
			html: '<div class="preview-holder">' +
						'<div class="preview"></div>' +
					'</div>' +
					'<div class="name">' + Sb.Locale.get('widgetImagegalleryEmptyName') + '</div>' +
					'<div class="description">' + Sb.Locale.get('widgetImagegalleryEmptyDescription') + '</div>'
		});

		var buttonImg = new Ext.ux.form.FileUploadField({
			ctCls		: 'sb-imagegallery-upload-button',
			multiple	: true,
			accept		: 'image/*',
			regex		: Sb.Utils.getImageValidExtensionsRegex(),
			buttonOnly	: true,
			width		: 'auto',
			name		: 'imagegallery',
			customButtonCfg: {
				xtype: 'box',
				autoEl: {
					tag: 'table',
					html: '<tr><td>' + Sb.Locale.get('widgetImagegalleryEmptyPreviewText') + '</td></tr>'
				}
			},
			listeners	: {
				scope		: this,
				click		: function (e) { e.stopPropagation(); },
				fileselected: this.onImageUpload
			}
		});

		new Ext.form.FormPanel({
			cls			: 'sb-imagegallery-upload',
			renderTo	: emptyFrame.first().first(),
			fileUpload	: true,
			border		: false,
			hideLabels	: true,
			items		: [
				buttonImg
			]
		});

		emptyFrame.addClassOnOver('frame-hover');
		if (!this.model.isChangeAllowed()) {
			emptyFrame.mask().on('click', function () {
				var win = Sb.Editor.Ui.mainWindow;
				if (this.model.isNeedAuth()) {
					if ('picasa' == this.model.get('storageType')) {
						this.startEdit();
						Sb.requestGoogleAuthToken();
					}
				} else {
					win.restore();
					win.statusBar.setMessageWarning(Sb.Locale.get('widgetImagegalleryErrorNoAlbums'));
				}
			}, this);
			this.model.on('albumsload', function () {
				emptyFrame.unmask();
			}, this, {single: true});
		}
	},

	addReordering: function () {
		this.clearReordering();

		var widget = this,
			placeholder = this.widgetContentEl.first().createChild({
			cls: 'frame frame-placeholder'
		},  this.widgetContentEl.child('.frame-empty'));
		placeholder.setVisibilityMode(Ext.Element.DISPLAY);
		placeholder.hide();

		Ext.each(this.widgetContentEl.query('[id^="frame-"]'), function (el) {
			var dragSource = new Ext.dd.DragSource(el, {
				ddGroup: 'imagegallery-group-' + widget.id,
				onStartDrag: function () {
					widget.isImageDragging = true;
					Ext.get(this.getEl()).setStyle({
						position: 'absolute',
						'z-index': 1
					});
				},

				b4MouseDown: function (e) {
					var x = e.getPageX();
					var y = e.getPageY();
					this.autoOffset(x, y);
				},

				autoOffset: function (x, y) {
					x -= this.startPageX;
					y -= this.startPageY;
					this.setDelta(x, y);
				},

				onInvalidDrop: function() {
					this.finishDrop();
				},

				onValidDrop: function() {
					this.finishDrop();

					var imageEl = Ext.get(this.getEl()),
						imageId = imageEl.id.replace(/frame-(.*)/i, "$1"),
						prevEl = imageEl.prev('[id^="frame-"]'),
						nextEl = imageEl.next('[id^="frame-"]'),
						relativeId;

					if (prevEl) {
						relativeId = prevEl.id.replace(/frame-(.*)/i, "$1");
						widget.model.updateImageOrder(imageId, relativeId, 'after');
					} else if (nextEl) {
						relativeId = nextEl.id.replace(/frame-(.*)/i, "$1");
						widget.model.updateImageOrder(imageId, relativeId, 'before');
					} else {
						return;
					}

					widget.jImageGallery.updateSettings({store: {data: widget.model.getImages()}});
				},

				finishDrop: function () {
					var domEl = this.getEl();

					Ext.get(domEl)
						.setStyle({
							position : '',
							left  : '',
							top   : '',
							'z-index': ''
						})
						.insertBefore(placeholder);

					placeholder.hide();

					this.dragging = false;
					(function() {
						widget.isImageDragging = false; // see onBeforeOpenImage method
					}).defer(1);
				},

				getDragEl: function () {
					return this.getEl();
				},

				createFrame: Ext.emptyFn,

				showFrame: Ext.emptyFn
			});

			dragSource.scroll = false;
			widget.imagesDD.push(dragSource);
		});

		this.imagesDropTarget = new Ext.dd.DropTarget(this.widgetContentEl, {
			ddGroup: 'imagegallery-group-' + widget.id,
			notifyDrop: function () {
				return true;
			},

			notifyOver : function(source, e) {
				var xy = e.getXY(),
					x = xy[0],
					y = xy[1];

				Ext.each(widget.widgetContentEl.query('[id^="frame-"]'), function (domEl) {
					if (domEl == source.getEl()) {
						return true;
					}

					var el = Ext.get(domEl),
						elTop = el.getY(),
						elBottom = elTop + el.getHeight(),
						elLeft = el.getX(),
						elRight = elLeft + el.getWidth(),
						elMiddle = (elRight + elLeft) / 2;

					if (x >= elLeft && x <= elRight && y >= elTop && y <= elBottom) {
						placeholder.show();
						if (x < elMiddle) {
							placeholder.insertBefore(el);
						} else {
							placeholder.insertAfter(el);
						}
					}
				}, this);

				return this.dropAllowed;
			}
		});
	},

	clearReordering: function () {
		Ext.each(this.imagesDD, function(dragSource) {
			dragSource.destroy();
		});
		this.imagesDD = [];

		if (this.imagesDropTarget) {
			this.imagesDropTarget.destroy();
		}
	},

	/**
	 * Before image gallery open callback
	 * Used to don't open image preview when dragging image
	 *
	 * @param {Object} gallery Object with gallery properties and methods
	 */
	onBeforeOpenImage: function(gallery) {
		return !this.isImageDragging;
	},

	/**
	 * On image gallery open callback
	 *
	 * @param {Object} gallery Object with gallery properties and methods
	 */
	onOpenImage: function (gallery) {
		var model	= this.model,
			manager = Sb.Editor.windowManager,
			remove = Ext.get('imageGalleryRemoveButton');

		remove.on('click', function () {
			this.onImageRemove(gallery.currentImage.id, jQuery.imagegallery.close);
		}, this);

		var clickOutsideEdit = function (el, event) {
			if (event.getTarget('#' + remove.id)) {
				return false;
			}
		};
		var beforeStartEdit = function (el, event) {
			if (this.model.isNeedAuth()) {
				if ('picasa' == this.model.get('storageType')) {
					this.startEdit();
					Sb.requestGoogleAuthToken(function () {
						el.startEdit();
					});
					return false;
				}
			}
		};

		this.title = new Sb.editor.component.Text({
			renderTo		: Ext.get(gallery.titleContainer[0]).update(''),
			maxTextLength	: 256,
			hasEditPanel	: false,
			hasControls		: false,
			isMultiline		: false,
			movable			: false,
			removable		: false,
			emptyText		: Sb.Locale.get('widgetImagegalleryDefaultImageTitle'),
			defaultText		: Sb.Locale.get('widgetImagegalleryEmptyName')
		});
		this.title.on('beforestartedit', beforeStartEdit, this);
		this.title.on('clickoutsideedit', clickOutsideEdit);
		this.title.on('completeedit', function () {
			var content = this.title.data.content ?
					this.title.data.content.replace(/[\\\/]/g, '_') :
					this.title.defaultText;
			if (model.updateImageProperty(gallery.currentImage.id, 'title', content)) {
				gallery.currentImage.title = content;
			}
		}, this);
		gallery.titleContainer[0] = this.title.contentElement.dom;
		manager.register(this.title, 9);

		this.description = new Sb.editor.component.Text({
			renderTo		: Ext.get(gallery.descriptionContainer[0]).update(''),
			maxTextLength	: 255,
			hasEditPanel	: false,
			hasControls		: false,
			movable			: false,
			removable		: false,
			emptyText		: Sb.Locale.get('widgetImagegalleryDefaultImageDescription')
		});
		this.description.on('beforestartedit', beforeStartEdit, this);
		this.description.on('clickoutsideedit', clickOutsideEdit);
		this.description.on('completeedit', function () {
			var content = Ext.util.Format.htmlDecode(
				this.description.data.content.replace(/(\r\n)?<br\s*\/?>/img, "\n").replace(/&nbsp;/img, ' ')
			);
			if (model.updateImageProperty(gallery.currentImage.id, 'description', content)) {
				gallery.currentImage.description = content;
				this.syncIfLocal();
			}
		}, this);
		gallery.descriptionContainer[0] = this.description.contentElement.dom;
		manager.register(this.description, 9);

		Ext.get('page').parent().setDisplayed(false);
		Sb.Editor.Ui.mainWindow.el.setDisplayed(false);
	},

	/**
	 * On image gallery close picture callback
	 *
	 * @param {Object} gallery Object with gallery properties and methods
	 */
	onCloseImage: function (gallery) {
		Sb.Editor.Ui.mainWindow.el.setDisplayed(true);
		Ext.get('page').parent().setDisplayed(true);
		this.jImageGallery.updateSettings({store: {data: this.model.getImages()}});
	},

	/**
	 * On image gallery change picture callback
	 *
	 * @param {Object} gallery Object with gallery properties and methods
	 */
	onChangeImage: function (gallery) {
		var format	= Ext.util.Format,
			image	= gallery.currentImage;

		this.title.setContent(image.title);
		this.description.setContent(format.nl2br(format.htmlEncode(image.description)));
	},

	onContentElementClick: function (event) {
		if (event.getTarget('.sb-imagegallery-upload-button')) {
			event.stopPropagation();
		} else {
			event.stopEvent();
		}
		if (this.afterDD) {
			this.afterDD = false;
			return;
		}
		this.startEdit();
	},

	onCancelEdit: function () {
		Sb.widget.Imagegallery.superclass.onCancelEdit.call(this);
		this.widgetContentEl.setHeight();
		this.isShowSaveMsg = false;
		var m = this.model;
		if (m.isRemote()) {
			var a = m.get('albumId');
			this.updateToLoading();
			if (a) {
				this.model.refreshImages(a, this.onImagesRefresh, this.onError, this);
			} else {
				this.onImagesRefresh();
			}
			m.save({
				success: function () {
					this.isShowSaveMsg = true;
				}.createDelegate(this)
			});
		} else {
			if (m.isLocal()) {
				m.loadImages(null, Ext.emptyFn);
			}
			this.refresh();
		}
	},

	onDocumentClick: function (event) {
		if (event.getTarget('.x-combo-list') || event.getTarget('.sb-dialog-imagegallery-albumcreate')) {
			return;
		}

		Sb.widget.Imagegallery.superclass.onDocumentClick.call(this, event);
	},

	onCompleteEdit: function () {
		this.model.saveIfChanged(this.memento, {
			success: function (result) {
				if (!result.success) {
					Ext.Msg.show({
						width: 800,
						title: Sb.Locale.get('errorMessageBoxTitle'),
						msg: result.message,
						closable: false,
						buttons: Ext.Msg.OK,
						icon: Ext.Msg.ERROR
					});
				}
			}
		});
	},

	onThumbSizeChange: function (value, oldValue) {
		this.jImageGallery.updateSettings({thumbSize: value, store: {currentPage: this.currentPage}});
		this.reanchorEditPanel();
	},

	onAlignChange: function (value) {
		this.widgetTitle.contentElement.setStyle('text-align', value);
		this.jImageGallery.updateSettings({align: value, store: {currentPage: this.currentPage}});
		this.reanchorEditPanel();
	},

	/**
	 * Image change handler, that fires up when user select image in upload control
	 */
	onImageUpload: function (field, fieldValue) {
		var frm = field.ownerCt.getForm(),
			win = Sb.Editor.Ui.mainWindow;
		if ('' === fieldValue) {
			return;
		}
		// :INFO: austimenko@ 20110606: Validate on client only when upload one file
		if (!Ext.isArray(fieldValue) && !frm.isValid()) {
			win.restore();
			win.statusBar.setMessageWarning(Sb.Locale.get('imageEditDialogChangeWarningInvalidType'));
			return;
		}
		/**
		 * Infroms user about problem during image(-s) uploading:
		 *    - in two levels (warning or error)
		 *    - and with details, when we have them
		 *
		 * @param {String} level warning or error
		 * @param {Array} details
		 */
		var informAboutProblem = function (level, details) {
			var msg	= Sb.Locale.get('widgetImagegalleryUploadingProblemMessage'),
				win	= Sb.Editor.Ui.mainWindow;
			if (details.length > 0) {
				Sb.widget.Imagegallery.showUploadProblemDetails = function () {
					(new Sb.Window({
						layout	: 'fit',
						width	: 500,
						height	: 300,
						plain	: true,
						title	: Sb.Locale.get('widgetImagegalleryUploadingProblemDetails'),
						modal	: true,
						items: {
							xtype		: 'textarea',
							value		: details.join('\n\n'),
							readOnly	: true
						}
					})).show();
				};
				msg += ' <a href="#" onclick="javascript:Sb.widget.Imagegallery.showUploadProblemDetails();">' +
					Sb.Locale.get('widgetImagegalleryUploadingProblemSeeDetails') + '</a>';
			}

			win.restore();
			win.statusBar['error' == level ? 'setMessageError' : 'setMessageWarning'](msg);
		};

		var url = Sb.Config.baseUrl + '/Upload/Imagegallery/',
			waitMsg = Sb.Locale.get('uploadingImage'),
			errors = [],
			params = {
				uuid: this.model.get('uuid'),
				storageType: this.model.get('storageType'),
				albumId: this.model.get('albumId'),
				storageAuthInfo: Ext.encode(this.model.get('storageAuthInfo')),
				'X-CSRF-Token': Sb.Config.csrfToken
			};

		if (Ext.isIE) {
			frm.submit({
				clientValidation: false,
				url: url,
				params: params,
				waitMsg: waitMsg,
				success: function (fp, action) {
					this.afterUploadImages(action.result);
					field.resetOnUpload();
					if (action.result.errors.length > 0) {
						informAboutProblem('warning', action.result.errors);
					}
				}.createDelegate(this),
				failure: function (fp, action) {
					informAboutProblem('error', action.result.errors);
				}
			});
		} else {
			var uploader = new Ext.ux.uploader.Manager({
				url: url,
				waitMsg: waitMsg,
				params: params,
				items: field.getFiles(),
				listeners: {
					scope: this,
					itemuploadsuccess: function (item, info) {
						this.afterUploadImages(info.result);
					},
					itemuploadfailure: function (item, info) {
						if (info.result.errors && info.result.errors.length > 0) {
							errors = errors.concat(info.result.errors);
						} else {
							errors.push(info.message);
						}
					},
					completeupload: function () {
						field.resetOnUpload();
						if (errors.length > 0) {
							informAboutProblem('warning', errors);
						}
					}
				}
			});
			uploader.startUpload();
		}
	},

	afterUploadImages: function (result) {
		this.currentPage = this.lastPage;
		Ext.each(result.images, function (image) {
			this.model.addImage(image);
		}, this);
		this.onImagesRefresh();
	},

	/**
	 * Image remove handler, that fires up when user remove image
	 *
	 * @param {String} imageId Image id
	 * @param {Function} yesCallback Callback, that will be called when user choose "Yes"
	 */
	onImageRemove: function (imageId, yesCallback) {
		var finish = function () {
			var confirmationMsg = Sb.Locale.get(
				'picasa' == this.model.get('storageType')&& this.model.get('picasaIsNeedClean') ?
				'widgetImagegalleryRemovePicasaImageConfirmation' : 'widgetImagegalleryRemoveImageConfirmation'
			);
			Ext.Msg.show({
				title	: Sb.Locale.get('confirmRemoval'),
				msg		: confirmationMsg + '<br>' + Sb.Locale.get('continueQuestion'),
				closable: false,
				buttons	: Ext.Msg.YESNO,
				scope	: this,
				fn		: function (btn) {
					if (btn == 'yes') {
						this.model.removeImage(imageId.replace(/frame-(.*)/i, "$1"));
						this.model.saveIfChanged();
						this.jImageGallery.updateSettings({store: {data: this.model.getImages()}});
						Ext.callback(yesCallback);
					}
				}
			});
		}.createDelegate(this);

		if (this.model.isNeedAuth()) {
			if ('picasa' == this.model.get('storageType')) {
				this.startEdit();
				Sb.requestGoogleAuthToken(finish);
				return;
			}
		}

		finish();
	},

	/**
	 * Displays error message
	 * @param result
	 */
	onError: function (result) {
		this.widgetContentEl.setHeight();
		this.togglePlaceholder(true);
		this.update(this.placeholderTpl.apply({
			placeholderMsg: Sb.Locale.get(result.messageLocaleKey)
		}));
	},

	/**
	 * Syncronizes changes on current widget when edit panel is closed and we are local
	 */
	syncIfLocal: function () {
		if (this.model.isLocal() && Sb.Editor.Ui.editComponent != this) {
			this.model.save();
		}
	},

	onDestroy: function () {
		this.clearReordering();
		Sb.widget.Imagegallery.superclass.onDestroy.call(this);
	}
});