/**
 * Allows to edit Image Gallery widget.
 *
 * @class Sb.widget.ImageGalleryEditPanel
 * @extends Sb.widget.WidgetEditPanel
 */
Ext.define('Sb.widget.ImagegalleryEditPanel', {
	extend: 'Sb.widget.WidgetEditTabPanel',
	cls: 'sb-widget-edit-panel-imagegallery',

	initComponent: function () {
		var items = [];

		if (Sb.Config.picasaEnabled) {
			items.push(this.createStoragePanel());
		}
		items.push({
			ref: '../appearanceTab',
			itemId: 'appearance',
			title: Sb.Locale.get('widgetImagegalleryAppearanceTabTitle'),
			autoHeight	: true,
			cls			: 'sb-widget-imagegallery-appearance-panel',
			layout: 'column',
			defaults: {
				layout: 'form',
				labelAlign	: 'top'
			},
			items: [{
				columnWidth: 0.6,
				items: [{
					ref: '../../../thumbSizeRadioGroup',
					xtype: 'radiogroup',
					fieldLabel: Sb.Locale.get('widgetImagegalleryThumbSizeFieldTitle'),
					cls: 'sb-radiogroup-inline sb-imagegallery-thumbnail-size',
					defaults: {
						xtype: 'radio',
						name: 'thumbSize'
					},
					items: [{
						boxLabel: Sb.Locale.get('widgetImagegalleryThumbSizeSmall'),
						inputValue: 'small'
					}, {
						boxLabel: Sb.Locale.get('widgetImagegalleryThumbSizeNormal'),
						inputValue: 'normal'
					}, {
						boxLabel: Sb.Locale.get('widgetImagegalleryThumbSizeLarge'),
						inputValue: 'large'
					}]
				}, {
					ref: '../../../alignRadioGroup',
					xtype: 'radiogroup',
					fieldLabel: Sb.Locale.get('widgetImagegalleryAlignFieldTitle'),
					cls: 'sb-radiogroup-inline sb-imagegallery-align',
					defaults: {
						xtype: 'radio',
						name: 'align'
					},
					items: [{
						boxLabel: Sb.Locale.get('widgetImagegalleryAlignLeft'),
						inputValue: 'left'
					}, {
						boxLabel: Sb.Locale.get('widgetImagegalleryAlignCenter'),
						inputValue: 'center'
					}, {
						boxLabel: Sb.Locale.get('widgetImagegalleryAlignRight'),
						inputValue: 'right'
					}]
				}]
			}, {
				columnWidth: 0.4,
				items: {
					ref: '../../../pageSize',
					xtype: 'numberfield',
					fieldLabel: Sb.Locale.get('widgetImageGalleryPageSizeFieldTitle'),
					labelWidth: 200,
					allowBlank: false,
					allowDecimals: false,
					minValue : 1,
					maxValue : 100,
					width: 32,
					msgTarget: 'under'
				}
			}],
			listeners: {
				scope: this,
				activate: this.initAppearanceTab
			}
		});
		Ext.apply(this, {
			items: items
		});

		Sb.widget.ImagegalleryEditPanel.superclass.initComponent.apply(this, arguments);
	},

	show: function () {
		this.loadedTabs = {};
		Sb.widget.ImagegalleryEditPanel.superclass.show.apply(this, arguments);
		if (this.widget.model.isNeedAuth() && this.storagePanel) {
			this.tabs.setActiveTab('storage');
		}
	},

	initAppearanceTab: function () {
		this.reanchor();

		if (this.loadedTabs.appearance) {
			return;
		}
		this.loadedTabs.appearance = true;

		this.bindModel({
			thumbSize: "thumbSizeRadioGroup",
			align: 'alignRadioGroup',
			pageSize: "pageSize"
		});
	},

	/**
	 * Generate storage tab
	 *
	 * @return {Ext.Panel}
	 */
	createStoragePanel: function () {
		/**
		 * Image storage tab
		 *
		 * @type Sb.form.FieldGroup
		 */
		this.storagePanel = new Sb.form.FieldGroup({
			itemId			: 'storage',
			title			: Sb.Locale.get('widgetImagegalleryStorageTabTitle'),
			helperMessage	: Sb.Locale.get('widgetImagegalleryStorageTabHelper'),
			layout			: 'column',
			cls				: 'sb-widget-imagegallery-storage-panel',
			labelAlign		: 'top',
			autoHeight		: true
		});
		this.storagePanel.on('render', this.renderStoragePanel, this, {single: true});
		return this.storagePanel;
	},

	/**
	 * Render storage tab
	 */
	renderStoragePanel: function () {
		/**
		 * Possible storage type values
		 *
		 * @type Array
		 * @property storageTypeValues
		 */
		this.storageTypeValues = ['local', 'picasa'];

		this.storagePanel.add([
			this.createStorageTypeSelector(),
			this.createStorageTypeContent()
		]);

		this.storagePanel.onActualShow(this.bindStoragePanel, this, this);
	},

	/**
	 * Create storage type selector
	 * @return Sb.form.FieldGroup
	 */
	createStorageTypeSelector: function () {
		var fieldGroup = new Sb.form.FieldGroup({
			columnWidth		: 0.25,
			autoWidth		: false,
			autoHeight		: true
		});
		var items = [];
		Ext.each(this.storageTypeValues, function (type, index) {
			var item = new Ext.form.Radio({
				boxLabel	: Sb.Locale.get('widgetImagegalleryStorageTabOption' + type.toUpperCaseFirst(), Sb.Config.general.product),
				inputValue	: type,
				name		: 'storageTypeItems',
				disabled	: type == 'local' && Sb.Editor.isDemoMode()
			});
			item.on('render', function () {
				item.wrap.unselectable();
			}, item, {single: true});
			items.push(item);
		});
		this.storageType = new Ext.form.RadioGroup({
			items			: items,
			columns			: 1,
			width			: Math.floor((this.width - 12) * 0.25 - 1),
			fieldLabel		: Sb.Locale.get('widgetImagegalleryStorageTabRadioLabel'),
			labelSeparator	: ''
		});
		this.storageType.on('change', this.setupStorageTypeContentGroup, this);
		fieldGroup.add(this.storageType);

		return fieldGroup;
	},

	/**
	 * Create storage type content
	 * @return Sb.form.FieldGroup
	 */
	createStorageTypeContent: function () {
		/**
		 * Storage type content group with panels in it, that will be switched by selector
		 *
		 * @type Sb.form.FieldGroup
		 * @property storageTypeContentGroup
		 */
		this.storageTypeContentGroup = new Sb.form.FieldGroup({
			layout			: 'card',
			columnWidth		: 0.75,
			autoWidth		: false,
			autoHeight		: true,
			items			: [
				this.createStorageTypeContentLocal(),
				this.createStorageTypeContentPicasa()
			]
		});

		return this.storageTypeContentGroup;
	},

	/**
	 * Create storage type content for Local storage
	 * @return Sb.form.FieldGroup
	 */
	createStorageTypeContentLocal: function () {
		return new Sb.form.FieldGroup({
			autoHeight	: true,
			cls			: 'sb-disconnected',
			html		: Sb.Locale.get('widgetImagegalleryStorageTabLocalNote', Sb.Config.general.product)
		});
	},

	/**
	 * Sets up tabs and activeItem
	 */
	setupStorageTypeContentGroup: function () {
		this.storageTypeContentGroup.layout.setActiveItem(
			this.storageTypeValues.indexOf(
				this.widget.model.get('storageType')
			)
		);
	},

	/**
	 * Get picasa`s login url
	 *
	 * @return String
	 */
	getPicasaLoginUrl: function () {
		return 'https://www.google.com/accounts/AuthSubRequest?session=1' +
			'&scope=' + encodeURIComponent('http://picasaweb.google.com/data') +
			'&next=' + encodeURIComponent(Sb.Config.originUrl + Sb.Config.baseUrl + '/js/externals/google/auth.html');
	},

	/**
	 * Create storage type content for Picasa
	 * @return Sb.form.FieldGroup
	 */
	createStorageTypeContentPicasa: function () {
		var	panelText	= Sb.Locale.get('widgetImagegalleryStorageTabPicasaLogin').replace('{0}', 'javascript:Sb.requestGoogleAuthToken()'),
			fg			= new Sb.form.FieldGroup({
							layout			: 'card',
							activeItem		: 0,
							autoHeight		: true,
							items			: [
								{itemId: 'disconnected', html: panelText, cls: 'sb-disconnected'},
								{itemId: 'connected', labelAlign: 'top', layout: 'form', cls: 'sb-connected', autoHeight: true},
								{itemId: 'loading', html: Sb.Locale.get('widgetImagegalleryLoading'), cls: 'sb-disconnected'}
							]
						}),
			createAlbum	= new Ext.Button({
				itemId	: 'createAlbum',
				text	: Sb.Locale.get('widgetImagegalleryStorageTabPicasaCreateAlbum'),
				iconCls	: 'sb-add-icon',
				handler	: this.onAlbumCreateClick,
				scope	: this
			});

		/**
		 * Id of currently selected album
		 *
		 * @type Ext.form.ComboBox
		 * @property albumId
		 */
		this.albumId = new Ext.form.ComboBox({
			fieldLabel		: Sb.Locale.get('widgetImagegalleryStorageTabPicasaAlbums'),
			value			: '',
			store			: new Ext.data.ArrayStore({
				fields	: ['id', 'title'],
				data	: []
			}),
			valueField		: 'id',
			displayField	: 'title',
			emptyText		: Sb.Locale.get('widgetImagegalleryLoading'),
			valueNotFoundText: '',
			enableKeyEvents	: true,
			mode			: 'local',
			itemCls			: 'sb-imagegallery-albumid-control',
			height			: 25,
			flex			: 1
		});
		fg.get('connected').add({
			xtype: 'compositefield',
			items: [this.albumId, createAlbum]
		});

		/**
		 * Helper text to give user link to currently selected album
		 *
		 * @type Ext.Component
		 * @property manageAlbumLinkPanel
		 */
		this.manageAlbumLinkPanel = new Ext.Component({
			html		: Sb.Locale.get('widgetImagegalleryStorageTabPicasaAlbumViewManage')
							.replace('{0}', '//picasaweb.google.com/'),
			hidden		: true,
			hideMode	: 'visibility'
		});
		this.manageAlbumLinkPanel.on('afterrender', function (cmp) {
			/**
			 * Link, that user clicks when he wants to manage/view album in picasa
			 *
			 * @type Ext.Element
			 * @property manageAlbumLink
			 */
			this.manageAlbumLink = cmp.el.child('a');
		}, this);
		fg.get('connected').add(this.manageAlbumLinkPanel);

		/**
		 * Do we need delete images from storage?
		 *
		 * @type Ext.form.Checkbox
		 * @property picasaIsNeedClean
		 */
		this.picasaIsNeedClean = new Ext.form.Checkbox({
			itemCls		: 'sb-imagegallery-is-cleanup-control',
			hideLabel	: true,
			boxLabel	: Sb.Locale.get('widgetImagegalleryIsNeedCleanLabel'),
			hidden		: true,
			hideMode	: 'visibility'
		});
		this.picasaIsNeedClean.on('check', function (me, value) {
			if (!value || value == this.widget.model.get('picasaIsNeedClean')) {
				return;
			}
			Ext.Msg.show({
				title	: Sb.Locale.get('confirmChangesDialogTitle'),
				msg		: Sb.Locale.get('widgetImagegalleryIsNeedCleanConfirmation'),
				closable: false,
				buttons	: Ext.Msg.YESNO,
				scope	: this,
				fn		: function (btn) {
					if (btn == 'no') {
						me.setValue(false);
					}
				}
			});
		}, this);
		fg.get('connected').add(this.picasaIsNeedClean);

		var fgOnActualShow	= function () {
			var m			= this.widget.model,
				isNeedAuth	= m.isNeedAuth(),
				activeItem	= isNeedAuth ? 'disconnected' : 'connected';

			fg.layout.setActiveItem(activeItem);
			if (m.isLocal() || isNeedAuth) {
				return;
			}

			var albumIdValue = m.get('albumId');
			this.albumId.disable();
			this.albumId.emptyText = Sb.Locale.get('widgetImagegalleryLoading');
			this.albumId.setValue('');
			this.manageAlbumLinkPanel.hide();
			this.picasaIsNeedClean.hide();
			this.toggleStorageTypeControls(false);
			m.refreshAlbums(function () {
				var data = [];
				Ext.each(m.albums, function (row, index) {
					data.push([row.id, row.title]);
				}, this);
				this.albumId.store.loadData(data);
				if (data.length > 0) {
					if (albumIdValue && -1 == this.albumId.store.find('id', albumIdValue)) {
						this.albumId.emptyText = '';
						this.albumId.setValue('');
						Sb.Editor.Ui.mainWindow.statusBar.setMessageWarning(
							Sb.Locale.get('widgetImagegalleryErrorNoOrPrivateAlbum')
						);
					} else {
						if (!albumIdValue) {
							m.set('albumId', data[0][0]);
						}
						this.albumId.setValue(m.get('albumId'));
						this.setupAlbumLink();
					}
					this.albumId.enable();
				} else {
					this.albumId.emptyText = '';
					this.albumId.setValue('');
					Sb.Editor.Ui.mainWindow.statusBar.setMessageWarning(
						Sb.Locale.get('widgetImagegalleryErrorNoAlbums')
					);
				}
				this.toggleStorageTypeControls(true);
			}, this);
		}.createDelegate(this);
		fg.onActualShow(fgOnActualShow, this, this);
		this.storageType.on('change', fgOnActualShow, this);

		Sb.requestGoogleAuthToken = function (successCallback) {
			var model		= this.widget.model,
				openParams	= 'width=888,height=444,toolbar=0,location=0,scrollbars=0,resizable=0,status=0';
			window.open(this.getPicasaLoginUrl(), 'sbImageGaleryPicasaAuth', openParams);
			// :INFO: austimenko 20110411: Callback, that will be called from opened window on success return
			Sb.pushGoogleAuthToken = function (token) {
				this.toggleStorageTypeControls(false);
				fg.layout.setActiveItem('loading');
				model.processStorageAuthInfo({temporaryToken: decodeURIComponent(token)}, function (result) {
					if (!result.success) {
						Sb.Editor.Ui.mainWindow.statusBar.setMessageWarning(
							Sb.Locale.get('widgetImagegalleryStorageTabPicasaAuth' + result.messageLocaleKey)
						);
						jQuery.imagegallery.close();
						this.reanchor();
						this.toggleStorageTypeControls(true);
					}
					fgOnActualShow();
					if (result.success) {
						Ext.callback(successCallback);
					}
				}.createDelegate(this));
			}.createDelegate(this);
		}.createDelegate(this);

		return fg;
	},

	/**
	 * Toggles storage type choose controls between loading states to avoid data corruption
	 *
	 * @param {Boolean} state
	 */
	toggleStorageTypeControls: function (state) {
		if (undefined === this.storageType) {
			return;
		}
		if (state) {
			this.storageType.ownerCt.enable();
		} else {
			this.storageType.ownerCt.disable();
		}
	},

	/**
	 * Updates link to current album
	 */
	setupAlbumLink: function () {
		if (!this.manageAlbumLink) {
			return;
		}
		this.manageAlbumLink.set({href: this.widget.model.getCurrentAlbum().url});
		this.manageAlbumLinkPanel.show();
		this.picasaIsNeedClean.show();
	},

	/**
	 * Event, fired on album change on current widget model
	 */
	onAlbumChange: function () {
		this.setupAlbumLink();
		this.widget.updateToLoading();
	},

	/**
	 * Handler for case of creating new album in storage, that starts simple dialog
	 */
	onAlbumCreateClick: function () {
		var okHandler, w;

		okHandler = function () {
			var m = this.widget.model,
				albumTitle = w.get('albumTitle');

			if (!albumTitle.isValid()) {
				return;
			}
			this.albumId.disable();
			this.albumId.setValue('');
			m.createAlbum(albumTitle.getValue(), function (albumIdValue) {
				m.loadAlbums(function () {
					var data = [],
						oldValue = m.get('albumId');
					Ext.each(m.albums, function (row, index) {
						data.push([row.id, row.title]);
					}, this);
					this.albumId.store.loadData(data);
					this.albumId.setValue(albumIdValue);
					m.set('albumId', albumIdValue);
					this.albumId.enable();
				}, this);
			}, this);
			w.close();
		};

		w = new Sb.Window({
			labelAlign		: 'top',
			width			: 400,
			title			: Sb.Locale.get('widgetImagegalleryStorageTabPicasaAlbumProperties'),
			modal			: true,
			resizable		: false,
			cls				: 'sb-dialog-imagegallery-albumcreate',
			defaultButton	: 0,
			items			: {
				itemId			: 'albumTitle',
				allowBlank		: false,
				maxLength		: 100,
				stripCharsRe	: new RegExp('^\\s+$'),
				xtype			: 'textfield',
				anchor			: '100%',
				fieldLabel		: Sb.Locale.get('widgetImagegalleryStorageTabPicasaAlbumName'),
				msgTarget		: 'under',
				value			: Sb.Locale.get('widgetImagegalleryStorageTabPicasaAlbumDefaultName')
			},
			buttons			: [{
				text			: Sb.Locale.get('ok'),
				scope			: this,
				handler			: okHandler
			}, {
				text			: Sb.Locale.get('cancelButtonTitle'),
				handler			: function () {
					w.close();
				}
			}],
			layout			: 'form'
		});
		w.show();
	},

	/**
	 * Binds current widget`s model to storage panel
	 */
	bindStoragePanel: function () {
		var m = this.widget.model;
		this.bindModel(['storageType', 'albumId', 'picasaIsNeedClean']);
		this.setupStorageTypeContentGroup();
		if (!m.hasListener('albumchange')) {
			m.on('albumchange', this.onAlbumChange, this);
		}
	},

	/**
	 * Gets the x,y coordinates to align this element with another element with respect to empty frame
	 *
	 * @param {Mixed} element The element to align to.
	 * @param {String} position (optional, defaults to "tl-bl?") The position to align to.
	 * @param {Array} offsets (optional) Offset the positioning by [x, y]
	 * @return {Array} [x, y]
	 */
	getAlignToXY : function (element, position, offsets) {
		var xy			= Sb.widget.ImagegalleryEditPanel.superclass.getAlignToXY.apply(this, arguments),
			emptyFrame	= this.widget.widgetContentEl.child('.frame-empty', true);

		if (!emptyFrame) {
			return xy;
		}

		var emptyFrameR	= Ext.lib.Region.getRegion(emptyFrame),
			width		= this.el.getWidth(),
			height		= this.el.getHeight(),
			panelR		= new Ext.lib.Region(xy[1], xy[0] + width, xy[1] + height, xy[0]),
			overlapR	= panelR.intersect(emptyFrameR);

		if (overlapR) {
			xy[1] = emptyFrameR.top - height - 3;
		}

		return xy;
	}
});