/**
 * @class Sb.widget.MapEditPanel
 * @extends Sb.widget.WidgetEditPanel
 * Class for map widget edit panel
 */
Ext.define('Sb.widget.MapEditPanel', {
	extend: 'Sb.widget.WidgetEditPanel',

	initComponent: function () {
		var comboboxConfig = {
			mode: 'local',
			editable: false,
			triggerAction: 'all',
			lazyInit: false,
			displayField: 'display',
			valueField: 'value',
			autoSelect: true,
			width: 100
		};

		this.placeField = new Ext.form.TextField({
			fieldLabel: Sb.Locale.get('mapPlaceFieldLabel'),
			width: 240,
			itemCls: 'place-field',
			listeners: {
				scope: this,
				specialkey: function (field, e) {
					if (e.getKey() === e.ENTER) {
						this.onPlaceChange();
					}
				}
			}
		});

		this.apiKeyField = new Ext.form.TextField({
			fieldLabel: Sb.Locale.get('mapApiKeyFieldLabel'),
			width: 240,
			listeners: {
				scope: this,
				change: this.onApiKeyChange
			}
		});

		this.searchButton = new Ext.Button({
			text: Sb.Locale.get('mapSearchButtonText'),
			cls: 'search-button',
			scope: this,
			handler: this.onPlaceChange
		});

		this.heightCombobox = new Ext.form.ComboBox(Ext.apply({}, {
			fieldLabel: Sb.Locale.get('mapHeightComboboxLabel'),
			itemCls: 'height-combobox',
			store: new Ext.data.ArrayStore({
				fields: ['value', 'display'],
				data: [
					['250', Sb.Locale.get('mapHeightSmall')],
					['400', Sb.Locale.get('mapHeightMedium')],
					['600', Sb.Locale.get('mapHeightBig')]
				]
			}),
			listeners: {
				scope: this,
				select: this.onHeightChange
			}
		}, comboboxConfig));

		this.helper = new Ext.BoxComponent({
			cls: 'helper',
			html: Sb.Locale.get('mapHelper')
		});

		Ext.apply(this, {
			autoHeight: true,
			layout: 'form',
			labelAlign: 'top',
			cls: 'sb-widget-edit-panel-map',
			items: [
				this.apiKeyField, this.placeField, this.searchButton, this.heightCombobox, this.helper
			]
		});

		Sb.widget.MapEditPanel.superclass.initComponent.apply(this, arguments);
	},

	show: function (widget) {
		Sb.widget.MapEditPanel.superclass.show.call(this, widget);

		var place = widget.model.get('place'),
			apiKey = widget.model.get('apiKey') || '';

		if (place) {
			place = widget.isPlaceInCoordinates(place) ? '' : place;
		} else {
			place = widget.getDefaultLocation();
		}
		this.placeField.setRawValue(place);
		this.apiKeyField.setRawValue(apiKey);

		this.heightCombobox.setValue(widget.model.get('height'));
	},

	onPlaceChange: function () {
		this.widget.model.set('place', this.placeField.getValue());
	},

	onApiKeyChange: function () {
		this.widget.model.set('apiKey', this.apiKeyField.getValue());
	},

	onHeightChange: function (combo) {
		this.widget.model.set('height', combo.getValue());
	}
});