/**
 * @class Sb.widget.PagecontentContainer
 * @extends Sb.widget.Container
 * @constructor
 * @param {Object} config The config object
 * @xtype pagecontent_container_widget
 */
Ext.define('Sb.widget.PagecontentContainer', {
	extend: 'Sb.widget.Container',
	xtype: 'pagecontent_container_widget',
	cls: 'sb-pagecontent-container',

	afterRender: function () {
		Sb.widget.PagecontentContainer.superclass.afterRender.call(this, arguments);
		// :INFO: smikhalyev@ 20120229: Configures the padding for the target zone in px.
		// See setPadding method's description in ExtJs DOC
		this.dt.setPadding(-5, 0, -5, 0);
	},

	/**
	 * Ability to drop dragged widget in side relative widget under Drag
	 * @param {Sb.widgetType.Base} widgetType
	 * @return {Boolean}
	 */
	allowAddWidgetSide: function (widgetType) {
		if (!Sb.widget.PagecontentContainer.superclass.allowAddWidgetSide.call(this, widgetType)) {
			return false;
		}

		var blogPostWidget = this.items.find(function (widget) {
			return widget.model.get('type') == 'blog_post';
		}),
			isSidebar = ['column1', 'column2'].indexOf(this.model.get('region')) != -1;

		return blogPostWidget || isSidebar ? false : true;
	},

	/**
	 * Checks if we can drop to target/container
	 *
	 * @param {Ext.dd.DragSource} source
	 * @param {Event} e
	 * @return {Boolean}
	 */
	isDroppable: function (source, e) {
		var isDroppable = Sb.widget.PagecontentContainer.superclass.isDroppable.call(this, source, e),
			blogPostWidget = this.items.find(function (widget) {
				return widget.model.get('type') == 'blog_post';
			});

		if (blogPostWidget === null) {
			return isDroppable;
		}

		return isDroppable && ['sharethis', 'commenting'].indexOf(source.dragData.widgetType.name) != -1;
	}
});