/**
 * @class Sb.widget.SliderEditPanel
 * @extends Sb.widget.WidgetEditPanel
 * Class for slider widget edit panel
 */
Ext.define('Sb.widget.SliderEditPanel', {
	extend: 'Sb.widget.WidgetEditTabPanel',

	initComponent: function () {
		var addImageButton = new Ext.ux.form.FileUploadField({
				buttonText: Sb.Locale.get('widgetSliderAddImageTitle'),
				multiple: true,
				accept: 'image/*',
				regex: Sb.Utils.getImageValidExtensionsRegex(),
				buttonOnly: true,
				width: 'auto',
				name: 'slider',
				listeners: {
					scope: this,
					click: function (e) { e.stopPropagation(); },
					fileselected: this.onImageUpload
				}
			});

		this.effectCombobox = new Ext.form.ComboBox({
			mode: 'local',
			editable: false,
			triggerAction: 'all',
			lazyInit: false,
			displayField: 'display',
			valueField: 'value',
			autoSelect: true,
			fieldLabel: Sb.Locale.get('sliderEffectComboboxLabel'),
			store: new Ext.data.ArrayStore({
				fields: ['value', 'display'],
				data: [
					['random', Sb.Locale.get('sliderEffectRandom')],
					['sliceDownRight', Sb.Locale.get('sliderEffectSliceDownRight')],
					['sliceDownLeft', Sb.Locale.get('sliderEffectSliceDownLeft')],
					['sliceUpRight', Sb.Locale.get('sliderEffectSliceUpRight')],
					['sliceUpLeft', Sb.Locale.get('sliderEffectSliceUpLeft')],
					['sliceUpDown', Sb.Locale.get('sliderEffectSliceUpDown')],
					['sliceUpDownLeft', Sb.Locale.get('sliderEffectSliceUpDownLeft')],
					['fold', Sb.Locale.get('sliderEffectFold')],
					['fade', Sb.Locale.get('sliderEffectFade')],
					['boxRandom', Sb.Locale.get('sliderEffectBoxRandom')],
					['boxRain', Sb.Locale.get('sliderEffectBoxRain')],
					['boxRainReverse', Sb.Locale.get('sliderEffectBoxRainReverse')],
					['boxRainGrow', Sb.Locale.get('sliderEffectBoxRainGrow')],
					['boxRainGrowReverse', Sb.Locale.get('sliderEffectBoxRainGrowReverse')]
				]
			})
		});

		this.arrowsCheckbox = new Ext.form.Checkbox({
			hideLabel: true,
			boxLabel: Sb.Locale.get('sliderShowArrowsCheckboxLabel')
		});

		this.descriptionTextarea = new Ext.form.TextArea({
			disabled: true,
			fieldLabel: Sb.Locale.get('sliderDescriptionTextareaLabel'),
			maxLength: 255,
			listeners: {
				scope: this,
				'valid': function (f) {
					var images = this.imagesDataView.getSelectedRecords();
					if (!images.length) {
						return;
					}

					images[0].set('description', f.getValue());
				}
			}
		});

		this.imagesDataView = new Ext.DataView({
			height: 130,
			singleSelect: true,
			cls: 'sb-slider-images-list',
			overClass: 'x-view-over',
			itemSelector: 'div.sb-selectbox-list-item',
			listeners: {
				scope: this,
				click: function (dataView, index, node, e) {
					var images = this.widget.model.images,
						image = images.getAt(index);

					if (!image) {
						return;
					}

					if (e.getTarget('.sb-selectbox-list-item-delete')) {
						images.remove(image);
						dataView.select(index);
						return;
					}
					if (e.getTarget('.sb-selectbox-list-item-up') && index > 0) {
						images.remove(image);
						images.insert(index - 1, image);
						dataView.select(image);
						return;
					}
					if (e.getTarget('.sb-selectbox-list-item-down') && index < images.getCount() - 1) {
						images.remove(image);
						images.insert(index + 1, image);
						dataView.select(image);
						return;
					}
				},
				selectionchange: function (dataView) {
					var images = dataView.getSelectedRecords();
					if (!images.length) {
						this.descriptionTextarea.disable();
						this.descriptionTextarea.setValue('');
						this.imageLinkField.disable();
						this.imageLinkTypeField.disable();
						return;
					}

					this.descriptionTextarea.enable();
					this.descriptionTextarea.setValue(images[0].get('description'));
					this.imageLinkField.enable();
					this.imageLinkTypeField.enable();
					var isPage = images[0].get('link').match(/^page:\/\/([^\/]*)\/?$/);
					if (isPage) {
						this.imageLinkTypeField.items.itemAt(1).setValue(true);
						this.imageLinkField.setValue(isPage[1]);
					} else {
						this.imageLinkTypeField.items.itemAt(0).setValue(true);
						this.imageLinkField.setValue(Sb.Editor.model.rootPage.get('identity'));
					}

					jQuery('#' + this.widget.widgetContentEl.id + ' .nivo-control[rel=' +
						this.widget.model.images.indexOf(images[0]) + ']').click();
				}
			}
		});

		Ext.apply(this, {
			cls: 'sb-widget-edit-panel-slider',
			items: [{
				title: Sb.Locale.get('widgetSliderImagesTabTitle'),
				helperMessage: Sb.Locale.get('widgetSliderImagesTabHelperMessage'),
				layout: 'column',
				items: [{
					cls: 'sb-slider-images-tab-left',
					columnWidth: 1,
					items: [{
						cls: 'sb-slider-images-view',
						items: this.imagesDataView
					}, {
						xtype: 'form',
						fileUpload: true,
						border: false,
						hideLabels: true,
						items: addImageButton
					}]
				}, {
					width: 250,
					layout: 'form',
					labelAlign: 'top',
					defaults: {
						anchor: '100%'
					},
					items: [this.descriptionTextarea, {
						ref: '../../../imageLinkTypeField',
						xtype: 'radiogroup',
						hideLabel: true,
						columns: 1,
						defaults: {
							xtype: 'radio',
							name: 'linkType'
						},
						items: [{
							boxLabel: Sb.Locale.get('widgetSliderImageLinkNone'),
							listeners: {
								scope: this,
								check: function (radio, checked) {
									if (!checked) {
										return;
									}
									var images = this.imagesDataView.getSelectedRecords();
									if (!images.length) {
										return;
									}

									images[0].set('link', '');
								}
							}
						}, {
							boxLabel: Sb.Locale.get('widgetSliderImageLinkPage'),
							listeners: {
								scope: this,
								check: function (radio, checked) {
									if (!checked) {
										return;
									}
									var images = this.imagesDataView.getSelectedRecords();
									if (!images.length) {
										return;
									}

									images[0].set('link', 'page://' + this.imageLinkField.getValue());
								}
							}
						}]
					}, {
						ref: '../../../imageLinkField',
						xtype: 'sbpagecombobox',
						hideLabel: true,
						listeners: {
							scope: this,
							select: function (combo) {
								var images = this.imagesDataView.getSelectedRecords();
								if (!images.length) {
									return;
								}

								this.imageLinkTypeField.items.itemAt(1).setValue(true);
								images[0].set('link', 'page://' + combo.getValue());
							}
						}
					}]
				}],
				listeners: {
					scope: this,
					activate: this.initImagesTab
				}
			}, {
				title: Sb.Locale.get('widgetSliderSettingsTabTitle'),
				layout: 'column',
				defaults: {
					layout: 'form',
					labelAlign: 'top',
					defaults: {
						anchor: '95%'
					},
					columnWidth: 0.5
				},
				items: [{
					items: [this.effectCombobox, {
						ref: '../../../pauseTimeField',
						xtype: 'sliderfield',
						fieldLabel: Sb.Locale.get('widgetSliderPauseTimeFieldTitle'),
						value: 3000,
						increment: 500,
						minValue: 3000,
						maxValue: 7000,
						tipText: function(thumb) {
							return Ext.util.Format.number(thumb.value / 1000, '0.0');
						},
						listeners: {
							scope: this,
							afterrender: function(field) {
								field.slider.on('changecomplete', function () {
									this.widget.model.set('pauseTime', field.getValue());
								}, this);
							}
						}
					}, {
						xtype: 'box',
						autoEl: {
							tag: 'table',
							cls: 'sb-widget-edit-panel-slider-pause-time-field-hint',
							cn: {
								tag: 'tr',
								cn: [{
									tag: 'td',
									cls: 'sb-widget-edit-panel-slider-pause-time-field-hint-left',
									html: Sb.Locale.get('3seconds')
								}, {
									tag: 'td',
									cls: 'sb-widget-edit-panel-slider-pause-time-field-hint-right',
									html: Sb.Locale.get('7seconds')
								}]
							}
						}
					}]
				}, {
					items: [{
						ref: '../../../navigationField',
						xtype: 'radiogroup',
						fieldLabel: Sb.Locale.get('widgetSliderNavigationFieldTitle'),
						cls: 'sb-radiogroup-inline',
						defaults: {
							xtype: 'radio',
							name: 'navigation'
						},
						items: [{
							boxLabel: Sb.Locale.get('widgetSliderNavigationNone'),
							inputValue: 'none'
						}, {
							boxLabel: Sb.Locale.get('widgetSliderNavigationInside'),
							inputValue: 'inside'
						}, {
							boxLabel: Sb.Locale.get('widgetSliderNavigationOutside'),
							inputValue: 'outside'
						}]
					}, this.arrowsCheckbox]
				}]
			}]
		});

		Sb.widget.SliderEditPanel.superclass.initComponent.apply(this, arguments);
	},

	show: function (widget) {
		this.imagesDataView.tpl = new Ext.XTemplate(
			'<tpl for=".">' +
				'<div class="sb-selectbox-list-item sb-header-images-list-item" >' +
				'<div>' +
				'<img src="' + widget.model.getResourcesUrl() + '/{image}" height="57">' +
				'<div class="sb-selectbox-list-item-delete"><span>X</span></div>' +
				'<div class="sb-selectbox-list-item-up"><span>Up</span></div>' +
				'<div class="sb-selectbox-list-item-down"><span>Down</span></div>' +
				'</div>' +
				'</div>' +
			'</tpl>'
		);
		this.loadedTabs = {};
		Sb.widget.SliderEditPanel.superclass.show.call(this, widget);

		this.bindModel({
			effect: 'effectCombobox',
			arrows: 'arrowsCheckbox',
			navigation: 'navigationField'
		});
		this.pauseTimeField.setValue(widget.model.get('pauseTime'));

		var rootPageNode = this.imageLinkField.findNode('id', Sb.Editor.model.rootPage.get('identity'));
		if (rootPageNode && rootPageNode.loaded){
			rootPageNode.reload();
		}
	},

	initImagesTab: function () {
		if (!this.loadedTabs.images) {
			this.imagesDataView.setStore(this.widget.model.images);
			this.loadedTabs.images = true;
		}

		this.imagesDataView.select(0);
	},

	onImageUpload: function (field, fieldValue) {
		var frm = field.ownerCt.getForm(),
			win = Sb.Editor.Ui.mainWindow;
		if ('' === fieldValue) {
			return;
		}

		if (!Ext.isArray(fieldValue) && !frm.isValid()) {
			win.restore();
			win.statusBar.setMessageWarning(Sb.Locale.get('imageEditDialogChangeWarningInvalidType'));
			return;
		}
		/**
		 * Infroms user about problem during image(-s) uploading:
		 *    - in two levels (warning or error)
		 *    - and with details, when we have them
		 *
		 * @param {String} level warning or error
		 * @param {Array} details
		 */
		var informAboutProblem = function (level, details) {
			var msg = Sb.Locale.get('widgetImagegalleryUploadingProblemMessage'),
				win = Sb.Editor.Ui.mainWindow;
			if (details.length > 0) {
				Sb.widget.Slider.showUploadProblemDetails = function () {
					(new Sb.Window({
						layout: 'fit',
						width: 500,
						height: 300,
						plain: true,
						title: Sb.Locale.get('widgetImagegalleryUploadingProblemDetails'),
						modal: true,
						items: {
							xtype: 'textarea',
							value: details.join('\n\n'),
							readOnly: true
						}
					})).show();
				};
				msg += ' <a href="#" onclick="javascript:Sb.widget.Slider.showUploadProblemDetails();">' +
					Sb.Locale.get('widgetImagegalleryUploadingProblemSeeDetails') + '</a>';
			}

			win.restore();
			win.statusBar['error' == level ? 'setMessageError' : 'setMessageWarning'](msg);
		};

		var url = Sb.Config.baseUrl + '/Upload/Slider/',
			waitMsg = Sb.Locale.get('uploadingImage'),
			errors = [],
			params = {
				uuid: this.widget.model.get('uuid'),
				'X-CSRF-Token': Sb.Config.csrfToken
			};

		if (Ext.isIE) {
			frm.submit({
				clientValidation: false,
				url: url,
				params: params,
				waitMsg: waitMsg,
				success: function (fp, action) {
					this.afterImageUpload(action.result);
					field.resetOnUpload();
					if (action.result.errors.length > 0) {
						informAboutProblem('warning', action.result.errors);
					}
				}.createDelegate(this),
				failure: function (fp, action) {
					informAboutProblem('error', action.result.errors);
				}
			});
		} else {
			var uploader = new Ext.ux.uploader.Manager({
				url: url,
				waitMsg: waitMsg,
				params: params,
				items: field.getFiles(),
				listeners: {
					scope: this,
					itemuploadsuccess: function (item, info) {
						this.afterImageUpload(info.result);
					},
					itemuploadfailure: function (item, info) {
						if (info.result && info.result.errors.length > 0) {
							errors = errors.concat(info.result.errors);
						} else {
							errors.push(info.message);
						}
					},
					completeupload: function () {
						field.resetOnUpload();
						if (errors.length > 0) {
							informAboutProblem('warning', errors);
						} else {
							this.initImagesTab();
						}
					}
				}
			});
			uploader.startUpload();
		}
	},

	afterImageUpload: function (result) {
		this.widget.model.images.loadData(result.images, true);
	}
});