/**
 * @class Sb.widget.Video
 * @extends Sb.widget.Script
 * Class for video widget
 * @xtype video_widget
 */
Ext.define('Sb.widget.Video', {
	extend: 'Sb.widget.Script',
	xtype: 'video_widget',

	/**
	 * @cfg {Sb.model.VideoWidget} model
	 * The {@link Sb.model.VideoWidget} to bind this Widget to.
	 */

	cls			: 'sb-widget-video',

	onContentElementClick: function (event) {
		if (event.getTarget('a') || event.getTarget('object') || event.getTarget('embed')) {
			return true;
		}
		return Sb.widget.Video.superclass.onContentElementClick.apply(this, arguments);
	},

	/**
	 * Determine, whether we have flash enabled
	 * @return {Boolean}
	 */
	isFlashEnabled: function () {
		return swfobject.hasFlashPlayerVersion('1');
	},

	renderContent: function () {
		if (!this.isFlashEnabled()) {
			Sb.widget.Video.superclass.renderPlaceholder.call(this,
				Sb.Locale.get('videoWidgetNeedFlashText', 'http://get.adobe.com/flashplayer/')
			);
		} else if (!this.isValidCode()) {
			Sb.widget.Video.superclass.renderPlaceholder.call(this, Sb.Locale.get('videoWidgetIncorrectEmbedCode'));
		} else {
			Sb.widget.Script.superclass.renderContent.call(this);
		}
	},

	prepareData: function () {
		var code		= this.model.getCode(),
			isCustom	= (1 == this.model.get('isCustom')),
			reOpaque	= new RegExp('wmode.+opaque', 'i'),
			reIframe	= new RegExp('(<iframe)(.*)(>)', 'ig'),
			reSrc		= new RegExp('( src="[^"]+)(")', 'ig'),
			reParams	= new RegExp(' src="[^"]+\\?[^"]+"', 'ig'),
			isProtect	= (isCustom && !reOpaque.test(code)),
			isIframe	= (reIframe.test(code)),
			delimeter	= null;

		if (isProtect) {
			code = code.replace(new RegExp('(<embed)', 'ig'), '$1 wmode="opaque"').replace(
				new RegExp('(<\/object>)', 'ig'), '<param name="wmode" value="opaque"/>$1'
			);
			if (isIframe) {
				delimeter	= reParams.test(code) ? '&' : '?';
				code		= code.replace(reSrc, '$1' + delimeter + 'wmode=opaque$2');
			}
		}

		return {code: code};
	},

	isValidCode: function () {
		var code		= this.model.getCode(),
			reIframe	= new RegExp('(<iframe)(.*)(>)', 'ig'),
			reObject	= new RegExp('(<object)(.*)(>)', 'ig'),
			reEmbed		= new RegExp('(<embed)(.*)(>)', 'ig'),
			isIframe	= (reIframe.test(code)),
			isObject	= (reObject.test(code)),
			isEmbed		= (reEmbed.test(code));
		return isIframe || (isObject && isEmbed);
	},

	onCompleteEdit: function () {
		this.getEditPanel().urlText.validate();
		Sb.widget.Video.superclass.onCompleteEdit.call(this);
	}
});