/**
 * @class Sb.widgetType.EshopCatalog
 * @extends Sb.widgetType.Base
 */
Ext.define('Sb.widgetType.EshopCatalog', {
	extend: 'Sb.widgetType.Base',

	allowedColors: 'text,link,h1',

	constructor: function (name) {
		this.referenceWidgetTypes = [
			Sb.WidgetTypeManager.get('blog'),
			Sb.WidgetTypeManager.get('eshopBasket')
		];

		Sb.widgetType.EshopCatalog.superclass.constructor.call(this, name);
	},

	init: function () {
		this.cache.clear();
	},

	/**
	 * Gets type for creating child
	 * @return {String}
	 */
	getPhpType: function () {
		return 'eshop_catalog';
	},

	isDroppableTo: function (container) {
		return !container.isInSiteWideArea() &&
			container.get('type') === 'container_vertical' &&
			container.get('region') === 'content';
	},

	isAvailable: function () {
		var masterWidget;

		this.disableReason = '';

		if (this.cache.length > 0) {
			this.disableReason = 'installed';
			return false;
		}

		if (!Sb.Editor.siteView.currentPage) {
			return true;
		}

		masterWidget = Sb.Editor.siteView.currentPage.getMasterWidget();
		if (masterWidget) {
			this.disableReason = masterWidget.getType().name;
			return false;
		}

		return true;
	},

	/**
	 * Fires when number of header widgets in changed
	 */
	onCacheChange: function () {
		Sb.widgetType.EshopCatalog.superclass.onCacheChange.call(this);

		Ext.each(this.referenceWidgetTypes, function (widgetType) {
			widgetType.toggle(widgetType.isAvailable());
		});
	},

	/**
	 * return page identity with eshop catalog instance
	 */
	getPage: function () {
		if (this.cache.length === 0) {
			return null;
		}
		return this.cache.get(0).ownerPage;
	},

	getFontSchemeCssTemplate: function () {
		return new Ext.XTemplate([
			'.content .ecwid-productBrowser-head {',
				'font-size: {eshopCatalogCategoryTitleSize}px;',
				'font-family: {eshopCatalogCategoryTitleFamily};',
			'}',

			//Name of category in list of categories
			'.content div.ecwid-productBrowser-subcategories-categoryName span {',
				'font-size: {eshopCatalogLarge2FontSize}px;',
				'font-family: {eshopCatalogFontFamily};',
			'}',

			'.content .ecwid-productBrowser-categoryPath span.ecwid-productBrowser-categoryPath-categoryLabel {',
				'font-size: {eshopCatalogFontSize}px;',
				'font-family: {eshopCatalogFontFamily};',
			'}',

			'.content div.ecwid-productBrowser-categoryPath a, ',
			'.content div.ecwid-productBrowser-categoryPath a:active,',
			'.content div.ecwid-productBrowser-categoryPath a:visited {',
				'font-size: {eshopCatalogFontSize}px;',
				'font-family: {eshopCatalogFontFamily};',
			'}',

			'.content .ecwid-productBrowser-categoryPath span.ecwid-productBrowser-categoryPath-separator {',
				'font-size: {eshopCatalogFontSize}px;',
				'font-family: {eshopCatalogFontFamily};',
			'}',

			'.content .ecwid-minicart-mini-link, .content span.ecwid-minicart-mini-count {',
				'font-size: {eshopCatalogFontSize}px;',
				'font-family: {eshopCatalogFontFamily};',
			'}',

			'.content .ecwid-results-topPanel div.ecwid-results-topPanel-itemsCountLabel {',
				'font-size: {eshopCatalogSmallFontSize}px;',
				'font-family: {eshopCatalogFontFamily};',
			'}',

			'.content .ecwid-results-topPanel div.ecwid-results-topPanel-viewAsPanel-current {',
				'font-size: {eshopCatalogFontSize}px;',
				'font-family: {eshopCatalogFontFamily};',
			'}',

			'.content .ecwid-results-topPanel div.ecwid-results-topPanel-viewAsPanel-link {',
				'font-size: {eshopCatalogFontSize}px;',
				'font-family: {eshopCatalogFontFamily};',
			'}',

			'.content .ecwid-results-topPanel div.gwt-Label {',
				'font-size: {eshopCatalogFontSize}px;',
				'font-family: {eshopCatalogFontFamily};',
			'}',

			'.content div.ecwid-pager span {',
				'font-size: {eshopCatalogFontSize}px;',
				'font-family: {eshopCatalogFontFamily};',
			'}',

			'.content div.ecwid-pager span.ecwid-pager-link-enabled, ',
			'.content div.ecwid-pager span.ecwid-pager-link-enabled span {',
				'font-size: {eshopCatalogFontSize}px;',
				'font-family: {eshopCatalogFontFamily};',
			'}',

			//Grid view
			'.content div.ecwid-productBrowser-productsGrid div.ecwid-productBrowser-productNameLink a {',
				'font-size: {eshopCatalogLargeFontSize}px;',
				'font-family: {eshopCatalogFontFamily};',
			'}',
			'.content div.ecwid-productBrowser-productsGrid div.ecwid-productBrowser-sku {',
				'font-size: {eshopCatalogFontSize}px;',
				'font-family: {eshopCatalogFontFamily};',
			'}',
			'.content div.ecwid-productBrowser-productsGrid div.ecwid-productBrowser-price {',
				'font-size: {eshopCatalogLarge2FontSize}px;',
				'font-family: {eshopCatalogFontFamily};',
				'font-weight: bold;',
			'}',

			//List view
			'.content div.ecwid-productBrowser-productsList div.ecwid-productBrowser-productNameLink a {',
				'font-size: {eshopCatalogLargeFontSize}px;',
				'font-family: {eshopCatalogFontFamily};',
			'}',
			'.content div.ecwid-productBrowser-productsList div.ecwid-productBrowser-sku {',
				'font-size: {eshopCatalogFontSize}px;',
				'font-family: {eshopCatalogFontFamily};',
			'}',
			'.content div.ecwid-productBrowser-productsList div.ecwid-productBrowser-productsList-descr {',
				'font-size: {eshopCatalogFontSize}px;',
				'font-family: {eshopCatalogFontFamily};',
			'}',
			'.content div.ecwid-productBrowser-productsList div.ecwid-productBrowser-price {',
				'font-size: {eshopCatalogLargeFontSize}px;',
				'font-family: {eshopCatalogFontFamily};',
				'font-weight: bold;',
			'}',

			//Table view
			'.content div.ecwid-productBrowser-productsTable div.ecwid-productBrowser-productNameLink a, ',
			'.content div.ecwid-productBrowser-productsTable div.ecwid-productBrowser-sku, ',
			'.content div.ecwid-productBrowser-productsTable div.ecwid-productBrowser-price, ',
			'.content div.ecwid-productBrowser-productsTable-addToBagLink {',
				'font-size: {eshopCatalogFontSize}px;',
				'font-family: {eshopCatalogFontFamily};',
			'}',
			'.content div.ecwid-productBrowser-productsTable div.ecwid-productBrowser-price {',
				'font-weight: bold;',
			'}',

			'.content div.ecwid-productBrowser-details-descr {',
				'font-size: {eshopCatalogFontSize}px;',
				'font-family: {eshopCatalogFontFamily};',
			'}'
		]);
	},

	getColorSchemeCssTemplate: function () {
		return new Ext.XTemplate([
			'.content .ecwid-productBrowser-head {',
				'color: {eshopCatalogCategoryTitleColor};',
			'}',
			'.content div.ecwid-productBrowser-subcategories-categoryName span {',
				'color: {eshopCatalogCategoryNameColor};',
			'}',
			'.content .ecwid-productBrowser-categoryPath span.ecwid-productBrowser-categoryPath-categoryLabel {',
				'color: {eshopCatalogCategoryBreadCrumbsLabelColor};',
			'}',

			'.content div.ecwid-productBrowser-categoryPath a, ',
			'.content div.ecwid-productBrowser-categoryPath a:active,',
			'.content div.ecwid-productBrowser-categoryPath a:visited {',
				'color: {eshopCatalogCategoryBreadCrumbsLinkColor};',
			'}',
			'.content .ecwid-productBrowser-categoryPath span.ecwid-productBrowser-categoryPath-separator {',
				'color: {eshopCatalogCategoryBreadCrumbsLinkColor}',
			'}',
			'.content .ecwid-minicart-mini-link, .content span.ecwid-minicart-mini-count {',
				'color: {eshopCatalogCategoryBasketLinkColor};',
			'}',
			'.content .ecwid-results-topPanel div.ecwid-results-topPanel-itemsCountLabel {',
				'color: {eshopCatalogCategoryItemsCountLabelColor};',
			'}',
			'.content .ecwid-results-topPanel div.ecwid-results-topPanel-viewAsPanel-current {',
				'color: {eshopCatalogCategoryViewCurrentLinkColor};',
			'}',
			'.content .ecwid-results-topPanel div.ecwid-results-topPanel-viewAsPanel-link {',
				'color: {eshopCatalogCategoryViewLinkColor};',
			'}',
			'.content .ecwid-results-topPanel div.gwt-Label {',
				'color: {eshopCatalogCategoryViewLabelColor};',
			'}',
			'.content div.ecwid-pager span {',
				'color: {eshopCatalogPaginationColor};',
			'}',
			'.content div.ecwid-pager span.ecwid-pager-link-enabled, ',
			'.content div.ecwid-pager span.ecwid-pager-link-enabled span {',
				'color: {eshopCatalogPaginationLinkColor};',
			'}',


			//Grid And List view

			'.content div.ecwid-productBrowser-productsGrid div.ecwid-productBrowser-productNameLink a, ',
			'.content div.ecwid-productBrowser-productsGrid div.ecwid-productBrowser-productNameLink a:active, ',
			'.content div.ecwid-productBrowser-productsGrid div.ecwid-productBrowser-productNameLink a:visited, ',
			'.content div.ecwid-productBrowser-productsList div.ecwid-productBrowser-productNameLink a, ',
			'.content div.ecwid-productBrowser-productsList div.ecwid-productBrowser-productNameLink a:active, ',
			'.content div.ecwid-productBrowser-productsList div.ecwid-productBrowser-productNameLink a:visited {',
				'color: {eshopCatalogGoodTilteColor};',
			'}',
			'.content div.ecwid-productBrowser-productsGrid div.ecwid-productBrowser-sku, ',
			'.content div.ecwid-productBrowser-productsList div.ecwid-productBrowser-sku {',
				'color: {eshopCatalogGoodArticleColor};',
			'}',

			//Table view
			'.content div.ecwid-productBrowser-productsTable div.ecwid-productBrowser-productNameLink a, ',
			'.content div.ecwid-productBrowser-productsTable div.ecwid-productBrowser-productNameLink a:active, ',
			'.content div.ecwid-productBrowser-productsTable div.ecwid-productBrowser-productNameLink a:visited {',
				'color: {eshopCatalogGoodTilteTableViewColor};',
			'}',
			'.content div.ecwid-productBrowser-productsTable div.ecwid-productBrowser-sku {',
				'color: {eshopCatalogGoodArticleTableViewColor};',
			'}',
			'.content div.ecwid-productBrowser-productsTable-addToBagLink {',
				'color: {eshopCatalogGoodToBasketLinkColor};',
			'}',

			'.content div.ecwid-productBrowser-cart div.ecwid-productBrowser-price,',
			'.content div.ecwid-productBrowser-price {',
				'color: {eshopCatalogGoodPriceColor};',
			'}',
			'.content div.ecwid-productBrowser-details-descr,',
			'.content div.ecwid-productBrowser-productsList-descr {',
				'color: {eshopCatalogGoodDescriptionColor};',
			'}'
		]);
	},

	/**
	 * Generate colors for eshop category widget.
	 * @return {Object}
	 */
	getCssData: function () {
		var colors = {},
			fonts = Sb.Editor.model.design.data.font;

		Ext.iterate(Sb.Editor.model.design.data.fontColor, function (name, value) {
			colors[name] = Sb.Editor.model.design.getColor(value);
		});

		return {
			eshopCatalogFontSize							: fonts.textFontSize,
			eshopCatalogSmallFontSize						: Math.ceil(fonts.textFontSize * 0.8),
			eshopCatalogLargeFontSize						: Math.ceil(fonts.textFontSize * 1.2),
			eshopCatalogLarge2FontSize						: Math.ceil(fonts.textFontSize * 1.4),
			eshopCatalogLarge3FontSize						: Math.ceil(fonts.textFontSize * 2),
			eshopCatalogFontFamily							: fonts.textFont,

			eshopCatalogCategoryTitleColor					: colors.contentH1Color,
			eshopCatalogCategoryTitleSize					: fonts.h1FontSize,
			eshopCatalogCategoryTitleFamily					: fonts.h1Font,

			eshopCatalogCategoryNameColor					: colors.contentTextColor,
			eshopCatalogCategoryBreadCrumbsLabelColor		: colors.contentTextColor,
			eshopCatalogCategoryBreadCrumbsLinkColor		: colors.contentLinkColor,
			eshopCatalogCategoryBasketLinkColor				: colors.contentLinkColor,
			eshopCatalogCategoryItemsCountLabelColor		: colors.contentTextColor,
			eshopCatalogCategoryViewLabelColor				: colors.contentTextColor,
			eshopCatalogCategoryViewLinkColor				: colors.contentLinkColor,
			eshopCatalogCategoryViewCurrentLinkColor		: colors.contentTextColor,
			eshopCatalogPaginationLinkColor					: colors.contentLinkColor,
			eshopCatalogPaginationColor						: colors.contentTextColor,

			eshopCatalogGoodTilteColor						: colors.contentTextColor,
			eshopCatalogGoodArticleColor					: colors.contentTextColor,

			eshopCatalogGoodTilteTableViewColor				: (Ext.ux.util.Color.isLight(colors.contentTextColor) ? '#333333' : colors.contentTextColor),
			eshopCatalogGoodArticleTableViewColor			: (Ext.ux.util.Color.isLight(colors.contentTextColor) ? '#333333' : colors.contentTextColor),

			eshopCatalogGoodPriceColor						: '#FF0000',
			eshopCatalogGoodDescriptionColor				: colors.contentTextColor,
			eshopCatalogGoodToBasketLinkColor				: colors.contentLinkColor
		};
	}
});