/**
 * @class Ext.ux.jsonp.Request
 * <p>Starts and finishes jsonp request by creating script tag.</p>
 */
Ext.define('Ext.ux.jsonp.Request', {
	extend: 'Ext.util.Observable',

	url			: '',
	params		: {},
	success		: Ext.emptyFn,
	failure		: Ext.emptyFn,
	callback	: Ext.emptyFn,
	scope		: null,
	callbackKey	: 'callback',
	script		: null,
	timeout		: 30000,

	/**
	 * @param {Object} options
	 */
	constructor: function (options) {
		options	= options || {};
		Ext.apply(this, options);
		this.script = document.createElement('script');
		this.script.type = 'text/javascript';
	},

	/**
	 * @param {String} callbackValue
	 */
	start: function (callbackValue) {
		this.params[this.callbackKey] = callbackValue;
		this.script.src = this.url + '?' + Ext.urlEncode(this.params);
		document.getElementsByTagName('head')[0].appendChild(this.script);
	},

	/**
	 * @param {Object} json
	 */
	finish: function (json) {
		this.callback.apply(this.scope);
		if (json) {
			this.success.apply(this.scope, [json]);
		} else {
			this.failure.apply(this.scope);
		}
	},

	destroy: function () {
		document.getElementsByTagName('head')[0].removeChild(this.script);
	}
});
