/**
 * @class Ext.ux.uploader.Item
 * @extends Ext.util.Observable
 * @constructor
 */
Ext.define('Ext.ux.uploader.Item', {
	extend: 'Ext.util.Observable',

	STATUS_READY: 'ready',
	STATUS_UPLOADING: 'uploading',
	STATUS_UPLOADED: 'uploaded',
	STATUS_UPLOAD_ERROR: 'uploaderror',

	constructor: function (config) {
		Ext.ux.uploader.Item.superclass.constructor.call(this, config);

		this.status = this.STATUS_READY;
		this.fileObject = config.file;
		this.addEvents('changestatus');
	},

	reset: function () {
		this.uploadErrorMessage = '';
		this.setStatus(this.STATUS_READY);
	},

	getFileObject: function () {
		return this.fileObject;
	},

	getFilename: function () {
		return this.getProperty('name');
	},

	getSize: function () {
		return this.getProperty('size');
	},

	getType: function () {
		return this.getProperty('type');
	},

	getProperty: function (propertyName) {
		if (this.fileObject) {
			return this.fileObject[propertyName];
		}
		return null;
	},

	getStatus: function () {
		return this.status;
	},

	setStatus: function (status) {
		this.status = status;
		this.fireEvent('changestatus', this, status);
	},

	isReady: function () {
		return (this.status === this.STATUS_READY);
	},

	isUploaded: function () {
		return (this.status === this.STATUS_UPLOADED);
	},

	setUploaded: function () {
		this.setStatus(this.STATUS_UPLOADED);
	},

	isUploadError: function () {
		return (this.status === this.STATUS_UPLOAD_ERROR);
	},

	getUploadErrorMessage: function () {
		return this.uploadErrorMessage;
	},

	setUploadError: function (message) {
		this.uploadErrorMessage = message;
		this.setStatus(this.STATUS_UPLOAD_ERROR);
	},

	setUploading: function () {
		this.setStatus(this.STATUS_UPLOADING);
	}
});